@@ -22,7 +22,8 @@ gen('Cl', xcname='PBE', scalarrel=True, exx=True, yukawa_gamma=0.40)
 c = {'energy': 0.005, 'eigenstates': 1e-2, 'density': 1e-2}
 mol = Cluster(molecule('NaCl'))
 mol.minimal_box(5.0, h=h)
-calc = GPAW(txt='NaCl.txt', xc='LCY-PBE:omega=0.40:excitation=singlet',
+calc = GPAW(mode='fd', txt='NaCl.txt',
+            xc='LCY-PBE:omega=0.40:excitation=singlet',
             eigensolver=RMMDIIS(), h=h, occupations=FermiDirac(width=0.0),
             spinpol=False, convergence=c)
 mol.calc = calc
