@@ -5,8 +5,9 @@ from gpaw.response.qeh import BuildingBlock
 
 df = DielectricFunction(calc='WSe2_gs_fulldiag.gpw',
                         eta=0.001,
-                        domega0=0.05,
-                        omega2=10.0,
+                        frequencies={'type': 'nonlinear',
+                                     'domega0': 0.05,
+                                     'omega2': 10.0},
                         nblocks=8,
                         ecut=150,
                         truncation='2D')
