@@ -428,6 +428,7 @@ class Density:
         """
         if spos_ac is None:
             spos_ac = atoms.get_scaled_positions() % 1.0
+        spos_ch = []    # for coreholes
 
         # Refinement of coarse grid, for representation of the AE-density
         # XXXXXXXXXXXX think about distribution depending on gridrefinement!
@@ -458,6 +459,12 @@ class Density:
         else:
             raise NotImplementedError
 
+        if self.nspins > 1 and not skip_core:
+            # Support for corehole in spin-polarized system
+            spos_ch = []
+            n_ch_a = []
+            n_ch = []
+
         # Add corrections to pseudo-density to get the AE-density
         splines = {}
         phi_aj = []
@@ -475,6 +482,20 @@ class Density:
             phit_aj.append(phit_j)
             nc_a.append([nc])
             nct_a.append([nct])
+            if self.setups[a].data.has_corehole and not skip_core and \
+                    self.nspins > 1:
+                assert self.setups[a].data.lcorehole == 0
+                work_setup = self.setups[a].data
+                rmax = nc.get_cutoff()
+                # work_setup.phicorehole_g
+                phi_ch = work_setup.phicorehole_g
+                phi_ch = np.where(abs(phi_ch) < 1e-160, 0, phi_ch)
+                n_ch = np.dot(work_setup.fcorehole, phi_ch**2) / (4 * pi)
+                # n_ch[0] = n_ch[1] # ch is allready taylored
+                # fcorehole should divided by two - use scale for this
+                n_ch_spl = work_setup.rgd.spline(n_ch, rmax, points=1000)
+                n_ch_a.append([n_ch_spl])
+                spos_ch.append(spos_ac[a])
 
         # Create localized functions from splines
         phi = BasisFunctions(gd, phi_aj)
@@ -485,6 +506,9 @@ class Density:
         phit.set_positions(spos_ac)
         nc.set_positions(spos_ac)
         nct.set_positions(spos_ac)
+        if spos_ch:
+            nch = LFC(gd, n_ch_a)
+            nch.set_positions(spos_ch)
 
         I_sa = np.zeros((self.nspins, len(spos_ac)))
         a_W = np.empty(len(phi.M_W), np.intc)
@@ -517,6 +541,10 @@ class Density:
 
                     if not skip_core:
                         I_a[a] -= setup.Nc / self.nspins
+                        if self.setups[a].data.has_corehole and \
+                                self.nspins > 1:
+                            I_a[a] += pow(-1, s) * \
+                                self.setups[a].data.fcorehole / 2
 
                 rank = D_asp.partition.rank_a[a]
                 D_asp.partition.comm.broadcast(D_sp, rank)
@@ -543,7 +571,10 @@ class Density:
 
             if not skip_core:
                 nc.lfc.ae_core_density_correction(scale, n_sg[s], a_W, I_a)
-
+                # correct for ch here
+                if spos_ch:
+                    nch.lfc.ae_core_density_correction(- scale * pow(-1, s),
+                                                       n_sg[s], a_W, I_a)
             nct.lfc.ae_core_density_correction(-scale, n_sg[s], a_W, I_a)
             D_asp.partition.comm.sum(I_a)
             N_c = gd.N_c
