@@ -2,26 +2,23 @@ from math import pi
 
 import numpy as np
 
-from io import StringIO
-from gpaw.response.wstc import WignerSeitzTruncatedCoulomb as WSTC
+from gpaw.hybrids.wstc import WignerSeitzTruncatedCoulomb as WSTC
 
 
 def coulomb_interaction(omega, gd, kd):
     if omega:
         return ShortRangeCoulomb(omega)
-    # Wigner-Seitz truncated Coulomb:
-    output = StringIO()
-    coulomb = WSTC(gd.cell_cv, kd.N_c, txt=output)
-    coulomb.description = output.getvalue()
-    return coulomb
+    return WSTC(gd.cell_cv, kd.N_c)
 
 
 class ShortRangeCoulomb:
     def __init__(self, omega):
         self.omega = omega
-        self.description = (
-            f'Short-range Coulomb: erfc(omega*r)/r (omega = {omega} bohr^-1)')
 
+    def get_description(self):
+        return f'Short-range Coulomb: erfc(omega*r)/r (omega = {self.omega} ' \
+               'bohr^-1)'
+    
     def get_potential(self, pd):
         G2_G = pd.G2_qG[0]
         x_G = 1 - np.exp(-G2_G / (4 * self.omega**2))
