@@ -8,7 +8,6 @@ import numpy as np
 from ase.units import Hartree
 from ase.utils.timing import Timer
 
-import _gpaw
 import gpaw.mpi as mpi
 from gpaw.xc import XC
 from gpaw.lrtddft.excitation import Excitation, ExcitationList, get_filehandle
@@ -413,23 +412,6 @@ class LrTDDFT(ExcitationList):
             pass
 
 
-def d2Excdnsdnt(dup, ddn):
-    """Second derivative of Exc polarised"""
-    res = [[0, 0], [0, 0]]
-    for ispin in range(2):
-        for jspin in range(2):
-            res[ispin][jspin] = np.zeros(dup.shape)
-            _gpaw.d2Excdnsdnt(dup, ddn, ispin, jspin, res[ispin][jspin])
-    return res
-
-
-def d2Excdn2(den):
-    """Second derivative of Exc unpolarised"""
-    res = np.zeros(den.shape)
-    _gpaw.d2Excdn2(den, res)
-    return res
-
-
 class LrTDDFTExcitation(Excitation):
 
     def __init__(self, Om=None, i=None,
