@@ -9,52 +9,71 @@ import numpy as np
 from ase import Atoms
 from ase.units import Bohr, Ha
 from gpaw import __version__
-from gpaw.core.uniform_grid import UniformGridFunctions
+from gpaw.core import UGArray
 from gpaw.dos import DOSCalculator
+from gpaw.mpi import world, synchronize_atoms
 from gpaw.new import Timer, cached_property
 from gpaw.new.builder import builder as create_builder
 from gpaw.new.calculation import (DFTCalculation, DFTState,
                                   ReuseWaveFunctionsError, units)
 from gpaw.new.gpw import read_gpw, write_gpw
-from gpaw.new.input_parameters import InputParameters
+from gpaw.new.input_parameters import (DeprecatedParameterWarning,
+                                       InputParameters)
 from gpaw.new.logger import Logger
 from gpaw.new.pw.fulldiag import diagonalize
-from gpaw.new.xc import XCFunctional
+from gpaw.new.xc import create_functional
 from gpaw.typing import Array1D, Array2D, Array3D
 from gpaw.utilities import pack
 from gpaw.utilities.memory import maxrss
 
 
 def GPAW(filename: Union[str, Path, IO[str]] = None,
+         txt: str | Path | IO[str] | None = '?',
+         communicator=None,
          **kwargs) -> ASECalculator:
     """Create ASE-compatible GPAW calculator."""
-    params = InputParameters(kwargs)
-    txt = params.txt
     if txt == '?':
         txt = '-' if filename is None else None
-    world = params.parallel['world']
-    log = Logger(txt, world)
+
+    parallel = kwargs.get('parallel', {})
+    comm = parallel.pop('world', None)
+    if comm is None:
+        comm = communicator or world
+    else:
+        warnings.warn(('Please use communicator=... '
+                       'instead of parallel={''world'': ...}'),
+                      DeprecatedParameterWarning)
+    log = Logger(txt, comm)
 
     if filename is not None:
-        assert set(kwargs) <= {'txt', 'parallel', 'communicator'}, kwargs
-        atoms, calculation, params, _ = read_gpw(filename, log,
-                                                 params.parallel)
-        return ASECalculator(params, log, calculation, atoms)
+        if not {'parallel'}.issuperset(kwargs):
+            illegal = set(kwargs) - {'parallel'}
+            raise ValueError('Illegal arguments when reading from a file: '
+                             f'{illegal}')
+        atoms, calculation, params, _ = read_gpw(filename,
+                                                 log=log,
+                                                 parallel=parallel)
+        return ASECalculator(params,
+                             log=log, calculation=calculation, atoms=atoms)
 
-    write_header(log, world, params)
-    return ASECalculator(params, log)
+    params = InputParameters(kwargs)
+    write_header(log, params)
+    return ASECalculator(params, log=log)
 
 
-def write_header(log, world, params):
+def write_header(log, params):
     from gpaw.io.logger import write_header as header
     log(f'#  __  _  _\n# | _ |_)|_||  |\n# |__||  | ||/\\| - {__version__}\n')
-    header(log, world)
+    header(log, log.comm)
     log('---')
     with log.indent('input parameters:'):
-        log(**{k: v for k, v in params.items()})
+        log(**dict(params.items()))
 
 
 def compare_atoms(a1: Atoms, a2: Atoms) -> set[str]:
+    if a1 is a2:
+        return set()
+
     if len(a1.numbers) != len(a2.numbers) or (a1.numbers != a2.numbers).any():
         return {'numbers'}
 
@@ -77,11 +96,13 @@ class ASECalculator:
 
     def __init__(self,
                  params: InputParameters,
+                 *,
                  log: Logger,
                  calculation=None,
                  atoms=None):
         self.params = params
         self.log = log
+        self.comm = log.comm
         self.calculation = calculation
 
         self.atoms = atoms
@@ -97,7 +118,9 @@ class ASECalculator:
         p = ', '.join(f'{key}: {val}' for key, val in params)
         return f'ASECalculator({p})'
 
-    def calculate_property(self, atoms: Atoms, prop: str) -> Any:
+    def calculate_property(self,
+                           atoms: Atoms | None,
+                           prop: str) -> Any:
         """Calculate (if not already calculated) a property.
 
         The ``prop`` string must be one of
@@ -109,6 +132,12 @@ class ASECalculator:
         * magmoms
         * dipole
         """
+        if atoms is None:
+            atoms = self.atoms
+        else:
+            synchronize_atoms(atoms, self.comm)
+        assert atoms is not None
+
         if self.calculation is not None:
             changes = compare_atoms(self.atoms, atoms)
             if changes & {'numbers', 'pbc', 'cell'}:
@@ -117,6 +146,7 @@ class ASECalculator:
                     magmom_a = self.calculation.results.get('magmoms')
                     if magmom_a is not None and magmom_a.any():
                         atoms = atoms.copy()
+                        assert atoms is not None  # MYPY: why is this needed?
                         atoms.set_initial_magnetic_moments(magmom_a)
 
                 if changes & {'numbers', 'pbc'}:
@@ -180,7 +210,7 @@ class ASECalculator:
     def create_new_calculation(self, atoms: Atoms) -> None:
         with self.timer('Init'):
             self.calculation = DFTCalculation.from_parameters(
-                atoms, self.params, self.log)
+                atoms, self.params, self.comm, self.log)
         self.atoms = atoms.copy()
 
     def create_new_calculation_from_old(self, atoms: Atoms) -> None:
@@ -217,34 +247,34 @@ class ASECalculator:
         return self.calculation.results['forces']
 
     def __del__(self):
+        self.log('---')
+        self.timer.write(self.log)
         try:
-            self.log('---')
-            self.timer.write(self.log)
             mib = maxrss() / 1024**2
             self.log(f'\nMax RSS: {mib:.3f}  # MiB')
-        except (NameError, AttributeError):
+        except NameError:
             pass
 
     def get_potential_energy(self,
-                             atoms: Atoms,
+                             atoms: Atoms | None = None,
                              force_consistent: bool = False) -> float:
         return self.calculate_property(atoms,
                                        'free_energy' if force_consistent else
                                        'energy')
 
-    def get_forces(self, atoms: Atoms) -> Array2D:
+    def get_forces(self, atoms: Atoms | None = None) -> Array2D:
         return self.calculate_property(atoms, 'forces')
 
-    def get_stress(self, atoms: Atoms) -> Array1D:
+    def get_stress(self, atoms: Atoms | None = None) -> Array1D:
         return self.calculate_property(atoms, 'stress')
 
-    def get_dipole_moment(self, atoms: Atoms) -> Array1D:
+    def get_dipole_moment(self, atoms: Atoms | None = None) -> Array1D:
         return self.calculate_property(atoms, 'dipole')
 
-    def get_magnetic_moment(self, atoms: Atoms) -> float:
+    def get_magnetic_moment(self, atoms: Atoms | None = None) -> float:
         return self.calculate_property(atoms, 'magmom')
 
-    def get_magnetic_moments(self, atoms: Atoms) -> Array1D:
+    def get_magnetic_moments(self, atoms: Atoms | None = None) -> Array1D:
         return self.calculate_property(atoms, 'magmoms')
 
     def write(self, filename, mode=''):
@@ -322,15 +352,16 @@ class ASECalculator:
     def get_effective_potential(self, spin=0):
         assert spin == 0
         vt_R = self.calculation.state.potential.vt_sR[spin]
-        return vt_R.to_pbc_grid().data * Ha
+        return vt_R.to_pbc_grid().gather(broadcast=True).data * Ha
 
     def get_electrostatic_potential(self):
         density = self.calculation.state.density
-        potential, vHt_x, W_aL = self.calculation.pot_calc.calculate(density)
-        if isinstance(vHt_x, UniformGridFunctions):
-            return vHt_x.to_pbc_grid().data * Ha
+        potential, _ = self.calculation.pot_calc.calculate(density)
+        vHt_x = potential.vHt_x
+        if isinstance(vHt_x, UGArray):
+            return vHt_x.gather(broadcast=True).to_pbc_grid().data * Ha
 
-        return vHt_x.interpolate(
+        return vHt_x.ifft(
             grid=self.calculation.pot_calc.fine_grid).data * Ha
 
     def get_atomic_electrostatic_potentials(self):
@@ -339,30 +370,44 @@ class ASECalculator:
     def get_electrostatic_corrections(self):
         return self.calculation.electrostatic_potential().atomic_corrections()
 
-    def get_pseudo_density(self, spin=None, gridrefinement=1):
+    def get_pseudo_density(self,
+                           spin=None,
+                           gridrefinement=1,
+                           broadcast=True) -> Array3D:
         assert spin is None
         nt_sr = self.calculation.densities().pseudo_densities(
             grid_refinement=gridrefinement)
-        return nt_sr.to_pbc_grid().data.sum(0)
+        return nt_sr.gather(broadcast=broadcast).data.sum(0)
 
     def get_all_electron_density(self,
                                  spin=None,
                                  gridrefinement=1,
+                                 broadcast=True,
                                  skip_core=False):
         assert spin is None
         n_sr = self.calculation.densities().all_electron_densities(
             grid_refinement=gridrefinement,
             skip_core=skip_core)
-        return n_sr.to_pbc_grid().data.sum(0)
+        return n_sr.gather(broadcast=broadcast).data.sum(0)
 
-    def get_eigenvalues(self, kpt=0, spin=0):
+    def get_eigenvalues(self, kpt=0, spin=0, broadcast=True):
         state = self.calculation.state
-        return state.ibzwfs.get_eigs_and_occs(k=kpt, s=spin)[0] * Ha
+        eig_n = state.ibzwfs.get_eigs_and_occs(k=kpt, s=spin)[0] * Ha
+        if broadcast:
+            if self.comm.rank != 0:
+                eig_n = np.empty(state.ibzwfs.nbands)
+            self.comm.broadcast(eig_n, 0)
+        return eig_n
 
-    def get_occupation_numbers(self, kpt=0, spin=0):
+    def get_occupation_numbers(self, kpt=0, spin=0, broadcast=True):
         state = self.calculation.state
         weight = state.ibzwfs.ibz.weight_k[kpt] * state.ibzwfs.spin_degeneracy
-        return state.ibzwfs.get_eigs_and_occs(k=kpt, s=spin)[1] * weight
+        occ_n = state.ibzwfs.get_eigs_and_occs(k=kpt, s=spin)[1] * weight
+        if broadcast:
+            if self.comm.rank != 0:
+                occ_n = np.empty(state.ibzwfs.nbands)
+            self.comm.broadcast(occ_n, 0)
+        return occ_n
 
     def get_reference_energy(self):
         return self.calculation.setups.Eref * Ha
@@ -378,6 +423,11 @@ class ASECalculator:
         state = self.calculation.state
         return state.ibzwfs.ibz.kpt_kc.copy()
 
+    def get_orbital_magnetic_moments(self):
+        """Return the orbital magnetic moment vector for each atom."""
+        from gpaw.new.orbmag import get_orbmag_from_calc
+        return get_orbmag_from_calc(self)
+
     def calculate(self, atoms, properties=None, system_changes=None):
         if properties is None:
             properties = ['energy']
@@ -407,7 +457,7 @@ class ASECalculator:
 
     @property
     def world(self):
-        return self.calculation.scf_loop.world
+        return self.comm
 
     @property
     def setups(self):
@@ -417,18 +467,37 @@ class ASECalculator:
     def initialized(self):
         return self.calculation is not None
 
+    def get_xc_functional(self):
+        return self.calculation.pot_calc.xc.name
+
     def get_xc_difference(self, xcparams):
         """Calculate non-selfconsistent XC-energy difference."""
-        state = self.calculation.state
-        xc = XCFunctional(xcparams, state.density.ncomponents)
-        exct = self.calculation.pot_calc.calculate_non_selfconsistent_exc(
-            state.density.nt_sR, xc)
+        dft = self.calculation
+        pot_calc = dft.pot_calc
+        state = dft.state
+        density = dft.state.density
+        xc = create_functional(xcparams, pot_calc.fine_grid)
+        if xc.type == 'MGGA' and density.taut_sR is None:
+            state.ibzwfs.make_sure_wfs_are_read_from_gpw_file()
+            if isinstance(state.ibzwfs.wfs_qs[0][0].psit_nX, SimpleNamespace):
+                params = InputParameters(dict(self.params.items()))
+                builder = create_builder(self.atoms, params, self.comm)
+                basis_set = builder.create_basis_set()
+                ibzwfs = builder.create_ibz_wave_functions(
+                    basis_set, state.potential, log=dft.log)
+                ibzwfs.fermi_levels = state.ibzwfs.fermi_levels
+                state.ibzwfs = ibzwfs
+                dft.scf_loop.update_density_and_potential = False
+                dft.converge()
+            density.update_ked(state.ibzwfs)
+        exct = pot_calc.calculate_non_selfconsistent_exc(
+            xc, density.nt_sR, density.taut_sR)
         dexc = 0.0
         for a, D_sii in state.density.D_asii.items():
             setup = self.setups[a]
             dexc += xc.calculate_paw_correction(
                 setup,
-                np.array([pack(D_ii) for D_ii in D_sii]))
+                np.array([pack(D_ii) for D_ii in D_sii.real]))
         return (exct + dexc - state.potential.energies['xc']) * Ha
 
     def diagonalize_full_hamiltonian(self,
@@ -436,12 +505,14 @@ class ASECalculator:
                                      scalapack=None,
                                      expert: bool = None) -> None:
         if expert is not None:
-            warnings.warn('Ignoring deprecated "expert" argument')
+            warnings.warn('Ignoring deprecated "expert" argument',
+                          DeprecationWarning)
         state = self.calculation.state
         ibzwfs = diagonalize(state.potential,
                              state.ibzwfs,
                              self.calculation.scf_loop.occ_calc,
-                             nbands)
+                             nbands,
+                             self.calculation.pot_calc.xc)
         self.calculation.state = DFTState(ibzwfs,
                                           state.density,
                                           state.potential)
@@ -453,21 +524,28 @@ class ASECalculator:
         from gpaw.response.groundstate import ResponseGroundStateAdapter
         return ResponseGroundStateAdapter(self)
 
-    def fixed_density(self, **kwargs):
+    def fixed_density(self, txt='-', **kwargs):
         kwargs = {**dict(self.params.items()), **kwargs}
         params = InputParameters(kwargs)
-        txt = params.txt
-        if txt == '?':
-            txt = '-'
-        world = params.parallel['world']
-        log = Logger(txt, world)
-        builder = create_builder(self.atoms, params)
+        log = Logger(txt, self.comm)
+        builder = create_builder(self.atoms, params, self.comm)
         basis_set = builder.create_basis_set()
         state = self.calculation.state
-        ibzwfs = builder.create_ibz_wave_functions(basis_set, state.potential,
+        comm1 = state.ibzwfs.kpt_band_comm
+        comm2 = builder.communicators['D']
+        potential = state.potential.redist(
+            builder.grid,
+            builder.electrostatic_potential_desc,
+            builder.atomdist,
+            comm1, comm2)
+        density = state.density.redist(builder.grid,
+                                       builder.interpolation_desc,
+                                       builder.atomdist,
+                                       comm1, comm2)
+        ibzwfs = builder.create_ibz_wave_functions(basis_set, potential,
                                                    log=log)
         ibzwfs.fermi_levels = state.ibzwfs.fermi_levels
-        state = DFTState(ibzwfs, state.density, state.potential)
+        state = DFTState(ibzwfs, density, potential)
         scf_loop = builder.create_scf_loop()
         scf_loop.update_density_and_potential = False
 
@@ -481,7 +559,10 @@ class ASECalculator:
 
         calculation.converge()
 
-        return ASECalculator(params, log, calculation, self.atoms)
+        return ASECalculator(params,
+                             log=log,
+                             calculation=calculation,
+                             atoms=self.atoms)
 
     def initialize(self, atoms):
         self.create_new_calculation(atoms)
@@ -515,3 +596,7 @@ class ASECalculator:
         """Create band-structure object for plotting."""
         from ase.spectrum.band_structure import get_band_structure
         return get_band_structure(calc=self)
+
+    @property
+    def symmetry(self):
+        return self.calculation.state.ibzwfs.ibz.symmetries.symmetry
