@@ -12,12 +12,13 @@ def create_basis(ibz,
                  setups,
                  dtype,
                  fracpos_ac,
-                 world=serial_comm,
-                 kpt_comm=serial_comm):
+                 comm=serial_comm,
+                 kpt_comm=serial_comm,
+                 band_comm=serial_comm):
     kd = KPointDescriptor(ibz.bz.kpt_Kc, nspins)
     kd.set_symmetry(SimpleNamespace(pbc=pbc_c),
                     ibz.symmetries.symmetry,
-                    comm=world)
+                    comm=comm)
     kd.set_communicator(kpt_comm)
 
     basis = BasisFunctions(grid._gd,
@@ -26,4 +27,8 @@ def create_basis(ibz,
                            dtype=dtype,
                            cut=True)
     basis.set_positions(fracpos_ac)
+    myM = (basis.Mmax + band_comm.size - 1) // band_comm.size
+    basis.set_matrix_distribution(
+        min(band_comm.rank * myM, basis.Mmax),
+        min((band_comm.rank + 1) * myM, basis.Mmax))
     return basis
