@@ -1,11 +1,13 @@
 from __future__ import annotations
 
-from gpaw.core import UniformGrid
-from gpaw.core.uniform_grid import UniformGridFunctions
-from gpaw.fd_operators import Laplace
-from gpaw.new import zip
+from gpaw.core import UGDesc
+from gpaw.core.arrays import DistributedArrays as XArray
+from gpaw.core.atom_arrays import AtomDistribution
+from gpaw.core.uniform_grid import UGArray
+from gpaw.fd_operators import Gradient, Laplace
+from gpaw.new import cached_property, zips
 from gpaw.new.builder import create_uniform_grid
-from gpaw.new.fd.pot_calc import UniformGridPotentialCalculator
+from gpaw.new.fd.pot_calc import FDPotentialCalculator
 from gpaw.new.hamiltonian import Hamiltonian
 from gpaw.new.poisson import PoissonSolver, PoissonSolverWrapper
 from gpaw.new.pwfd.builder import PWFDDFTComponentsBuilder
@@ -13,13 +15,25 @@ from gpaw.poisson import PoissonSolver as make_poisson_solver
 
 
 class FDDFTComponentsBuilder(PWFDDFTComponentsBuilder):
-    def __init__(self, atoms, params, nn=3, interpolation=3):
-        super().__init__(atoms, params)
+    def __init__(self, atoms, params, *, comm, nn=3, interpolation=3):
+        super().__init__(atoms,
+                         params,
+                         comm=comm)
         assert not self.soc
         self.kin_stencil_range = nn
         self.interpolation_stencil_range = interpolation
 
         self._nct_aR = None
+        self._tauct_aR = None
+
+        self.electrostatic_potential_desc = self.fine_grid
+        self.interpolation_desc = self.fine_grid
+
+    @cached_property
+    def atomdist(self) -> AtomDistribution:
+        return AtomDistribution(
+            self.grid.ranks_from_fractional_positions(self.fracpos_ac),
+            self.grid.comm)
 
     def create_uniform_grids(self):
         grid = create_uniform_grid(
@@ -35,15 +49,21 @@ class FDDFTComponentsBuilder(PWFDDFTComponentsBuilder):
         # decomposition=[2 * d for d in grid.decomposition]
         return grid, fine_grid
 
-    def create_wf_description(self) -> UniformGrid:
+    def create_wf_description(self) -> UGDesc:
         return self.grid.new(dtype=self.dtype)
 
     def get_pseudo_core_densities(self):
         if self._nct_aR is None:
             self._nct_aR = self.setups.create_pseudo_core_densities(
-                self.grid, self.fracpos_ac, atomdist=self.atomdist)
+                self.grid, self.fracpos_ac, atomdist=self.atomdist, xp=self.xp)
         return self._nct_aR
 
+    def get_pseudo_core_ked(self):
+        if self._tauct_aR is None:
+            self._tauct_aR = self.setups.create_pseudo_core_ked(
+                self.grid, self.fracpos_ac, atomdist=self.atomdist)
+        return self._tauct_aR
+
     def create_poisson_solver(self) -> PoissonSolver:
         solver = make_poisson_solver(**self.params.poissonsolver)
         solver.set_grid_descriptor(self.fine_grid._gd)
@@ -51,13 +71,11 @@ class FDDFTComponentsBuilder(PWFDDFTComponentsBuilder):
 
     def create_potential_calculator(self):
         poisson_solver = self.create_poisson_solver()
-        nct_aR = self.get_pseudo_core_densities()
-        return UniformGridPotentialCalculator(self.grid,
-                                              self.fine_grid,
-                                              self.setups,
-                                              self.xc, poisson_solver,
-                                              nct_aR, self.nct_R,
-                                              self.interpolation_stencil_range)
+        return FDPotentialCalculator(
+            self.grid, self.fine_grid, self.setups, self.xc, poisson_solver,
+            fracpos_ac=self.fracpos_ac, atomdist=self.atomdist,
+            interpolation_stencil_range=self.interpolation_stencil_range,
+            xp=self.xp)
 
     def create_hamiltonian_operator(self, blocksize=10):
         return FDHamiltonian(self.wf_desc, self.kin_stencil_range, blocksize)
@@ -93,11 +111,10 @@ class FDDFTComponentsBuilder(PWFDDFTComponentsBuilder):
             data = reader.wave_functions.proxy(name, *index)
             data.scale = c
             if self.communicators['w'].size == 1:
-                wfs.psit_nX = UniformGridFunctions(grid, self.nbands,
-                                                   data=data)
+                wfs.psit_nX = UGArray(grid, self.nbands, data=data)
             else:
                 band_comm = self.communicators['b']
-                wfs.psit_nX = UniformGridFunctions(
+                wfs.psit_nX = UGArray(
                     grid, self.nbands,
                     comm=band_comm)
                 if grid.comm.rank == 0:
@@ -119,11 +136,38 @@ class FDHamiltonian(Hamiltonian):
         self._gd = grid._gd
         self.kin = Laplace(self._gd, -0.5, kin_stencil, grid.dtype)
 
-    def apply(self, vt_sR, psit_nR, out, spin):
-        self.kin.apply(psit_nR.data, out.data, psit_nR.desc.phase_factors_cd)
-        for p, o in zip(psit_nR.data, out.data):
-            o += p * vt_sR.data[spin]
-        return out
+        # For MGGA:
+        self.grad_v = []
+
+    def apply_local_potential(self,
+                              vt_R: UGArray,
+                              psit_nR: XArray,
+                              out: XArray,
+                              ) -> None:
+        assert isinstance(psit_nR, UGArray)
+        assert isinstance(out, UGArray)
+        self.kin(psit_nR, out)
+        for p, o in zips(psit_nR.data, out.data):
+            o += p * vt_R.data
+
+    def apply_mgga(self,
+                   dedtaut_R: UGArray,
+                   psit_nR: XArray,
+                   vt_nR: XArray) -> None:
+        if len(self.grad_v) == 0:
+            grid = psit_nR.desc
+            self.grad_v = [
+                Gradient(grid._gd, v, n=3, dtype=grid.dtype)
+                for v in range(3)]
+
+        tmp_R = psit_nR.desc.empty()
+        for psit_R, out_R in zips(psit_nR, vt_nR):
+            for grad in self.grad_v:
+                grad(psit_R, tmp_R)
+                tmp_R.data *= dedtaut_R.data
+                grad(tmp_R, tmp_R)
+                tmp_R.data *= 0.5
+                out_R.data -= tmp_R.data
 
     def create_preconditioner(self, blocksize):
         from types import SimpleNamespace
@@ -132,7 +176,7 @@ class FDHamiltonian(Hamiltonian):
         pc = PC(self._gd, self.kin, self.grid.dtype, self.blocksize)
 
         def apply(psit, residuals, out):
-            kpt = SimpleNamespace(phase_cd=psit.desc.phase_factors_cd)
+            kpt = SimpleNamespace(phase_cd=psit.desc.phase_factor_cd)
             pc(residuals.data, kpt, out=out.data)
 
         return apply
