@@ -21,7 +21,7 @@ from gpaw.typing import DTypeLike
 
 ENERGY_NAMES = ['kinetic', 'coulomb', 'zero', 'external', 'xc', 'entropy',
                 'total_free', 'total_extrapolated',
-                'band']
+                'band', 'stress']
 
 
 def write_gpw(filename: str,
@@ -30,10 +30,10 @@ def write_gpw(filename: str,
               calculation: DFTCalculation,
               skip_wfs: bool = True) -> None:
 
-    world = params.parallel['world']
+    comm = calculation.comm
 
     writer: ulm.Writer | ulm.DummyWriter
-    if world.rank == 0:
+    if comm.rank == 0:
         writer = ulm.Writer(filename, tag='gpaw')
     else:
         writer = ulm.DummyWriter()
@@ -70,7 +70,7 @@ def write_gpw(filename: str,
                                         state.ibzwfs,
                                         state.density.nt_sR.desc)
 
-    world.barrier()
+    comm.barrier()
 
 
 def write_wave_function_indices(writer, ibzwfs, grid):
@@ -106,7 +106,9 @@ def write_wave_function_indices(writer, ibzwfs, grid):
 
 
 def read_gpw(filename: Union[str, Path, IO[str]],
+             *,
              log: Union[Logger, str, Path, IO[str]] = None,
+             comm=None,
              parallel: dict[str, Any] = None,
              dtype: DTypeLike = None):
     """
@@ -117,10 +119,11 @@ def read_gpw(filename: Union[str, Path, IO[str]],
     atoms, calculation, params, builder
     """
     parallel = parallel or {}
-    world = parallel.get('world', mpi.world)
 
     if not isinstance(log, Logger):
-        log = Logger(log, world)
+        log = Logger(log, comm or mpi.world)
+
+    comm = log.comm
 
     log(f'Reading from {filename}')
 
@@ -137,28 +140,36 @@ def read_gpw(filename: Union[str, Path, IO[str]],
 
     # kwargs['nbands'] = reader.wave_functions.eigenvalues.shape[-1]
 
-    params = InputParameters(kwargs, warn=False)
-    builder = create_builder(atoms, params)
+    for old_keyword in ['fixdensity', 'txt']:
+        kwargs.pop(old_keyword, None)
 
-    if world.rank == 0:
+    params = InputParameters(kwargs, warn=False)
+    builder = create_builder(atoms, params, comm)
+
+    if comm.rank == 0:
         nt_sR_array = reader.density.density * bohr**3
         vt_sR_array = reader.hamiltonian.potential / ha
+        if builder.xc.type == 'MGGA':
+            taut_sR_array = reader.density.ked * (bohr**3 / ha)
+            dedtaut_sR_array = reader.hamiltonian.mgga_potential * bohr**-3
         D_sap_array = reader.density.atomic_density_matrices
         dH_sap_array = reader.hamiltonian.atomic_hamiltonian_matrices / ha
         shape = nt_sR_array.shape[1:]
     else:
         nt_sR_array = None
         vt_sR_array = None
+        taut_sR = None
+        dedtaut_sR = None
         D_sap_array = None
         dH_sap_array = None
         shape = None
 
-    if builder.grid.global_shape() != mpi.broadcast(shape, comm=world):
+    if builder.grid.global_shape() != mpi.broadcast(shape, comm=comm):
         # old gpw-file:
         kwargs.pop('h', None)
         kwargs['gpts'] = nt_sR_array.shape[1:]
         params = InputParameters(kwargs, warn=False)
-        builder = create_builder(atoms, params)
+        builder = create_builder(atoms, params, comm)
 
     if dtype is not None:
         params.mode['dtype'] = dtype
@@ -169,6 +180,13 @@ def read_gpw(filename: Union[str, Path, IO[str]],
     nt_sR = builder.grid.empty(builder.ncomponents)
     vt_sR = builder.grid.empty(builder.ncomponents)
 
+    if builder.xc.type == 'MGGA':
+        taut_sR = builder.grid.empty(builder.ncomponents)
+        dedtaut_sR = builder.grid.empty(builder.ncomponents)
+    else:
+        taut_sR = None
+        dedtaut_sR = None
+
     atom_array_layout = AtomArraysLayout([(setup.ni * (setup.ni + 1) // 2)
                                           for setup in builder.setups],
                                          atomdist=builder.atomdist)
@@ -179,21 +197,41 @@ def read_gpw(filename: Union[str, Path, IO[str]],
     if kpt_band_comm.rank == 0:
         nt_sR.scatter_from(nt_sR_array)
         vt_sR.scatter_from(vt_sR_array)
+        if builder.xc.type == 'MGGA':
+            taut_sR.scatter_from(taut_sR_array)
+            dedtaut_sR.scatter_from(dedtaut_sR_array)
         D_asp.scatter_from(D_sap_array)
         dH_asp.scatter_from(dH_sap_array)
-
     if reader.version < 4:
         convert_to_new_packing_convention(D_asp, density=True)
         convert_to_new_packing_convention(dH_asp)
 
     kpt_band_comm.broadcast(nt_sR.data, 0)
     kpt_band_comm.broadcast(vt_sR.data, 0)
+    if builder.xc.type == 'MGGA':
+        kpt_band_comm.broadcast(taut_sR.data, 0)
+        kpt_band_comm.broadcast(dedtaut_sR.data, 0)
     kpt_band_comm.broadcast(D_asp.data, 0)
     kpt_band_comm.broadcast(dH_asp.data, 0)
 
-    density = Density.from_data_and_setups(nt_sR, D_asp.to_full(),
-                                           builder.params.charge,
-                                           builder.setups)
+    if reader.version >= 4:
+        if comm.rank == 0:
+            vHt_x_array = reader.hamiltonian.electrostatic_potential / ha
+        else:
+            vHt_x_array = None
+        vHt_x = builder.electrostatic_potential_desc.empty()
+        if kpt_band_comm.rank == 0:
+            vHt_x.scatter_from(vHt_x_array)
+        kpt_band_comm.broadcast(vHt_x.data, 0)
+    else:
+        vHt_x = None
+
+    density = Density.from_data_and_setups(
+        nt_sR, taut_sR, D_asp.to_full(),
+        builder.params.charge,
+        builder.setups,
+        builder.get_pseudo_core_densities(),
+        builder.get_pseudo_core_ked())
     energies = {name: reader.hamiltonian.get(f'e_{name}', np.nan) / ha
                 for name in ENERGY_NAMES}
     penergies = {key: e for key, e in energies.items()
@@ -202,7 +240,8 @@ def read_gpw(filename: Union[str, Path, IO[str]],
     e_entropy = penergies.pop('entropy')
     penergies['kinetic'] -= e_band
 
-    potential = Potential(vt_sR, dH_asp.to_full(), penergies)
+    potential = Potential(vt_sR, dH_asp.to_full(), dedtaut_sR, penergies,
+                          vHt_x)
 
     ibzwfs = builder.read_ibz_wave_functions(reader)
     ibzwfs.energies = {
@@ -226,9 +265,9 @@ def read_gpw(filename: Union[str, Path, IO[str]],
 
     calculation.results = results
 
-    if builder.mode in ['pw', 'fd']:
+    if builder.mode in ['pw', 'fd']:  # fd = finite difference
         data = ibzwfs.wfs_qs[0][0].psit_nX.data
-        if not hasattr(data, 'fd'):
+        if not hasattr(data, 'fd'):  # fd = file descriptor
             reader.close()
     else:
         reader.close()
