@@ -5,12 +5,12 @@ from typing import Callable
 
 import numpy as np
 from ase.units import Ha
-from gpaw.core.arrays import DistributedArrays as DA
-from gpaw.core.atom_centered_functions import AtomArrays as AA
+from gpaw.core.arrays import DistributedArrays as XArray
+from gpaw.core.atom_centered_functions import AtomArrays
 from gpaw.core.matrix import Matrix
 from gpaw.gpu import as_xp
 from gpaw.mpi import broadcast_float
-from gpaw.new import zip
+from gpaw.new import zips
 from gpaw.new.calculation import DFTState
 from gpaw.new.eigensolver import Eigensolver
 from gpaw.new.hamiltonian import Hamiltonian
@@ -20,8 +20,6 @@ from gpaw.typing import Array1D, Array2D
 from gpaw.utilities.blas import axpy
 from gpaw.yml import obj2yaml as o2y
 
-AAFunc = Callable[[AA, AA], AA]
-
 
 class Davidson(Eigensolver):
     def __init__(self,
@@ -54,10 +52,11 @@ class Davidson(Eigensolver):
         assert isinstance(wfs, PWFDWaveFunctions)
         xp = wfs.psit_nX.xp
         B = ibzwfs.nbands
+        b = max(wfs.n2 - wfs.n1 for wfs in ibzwfs)
         domain_comm = wfs.psit_nX.desc.comm
         band_comm = wfs.band_comm
         shape = ibzwfs.get_max_shape()
-        shape = (2, B) + shape
+        shape = (2, b) + shape
         dtype = wfs.psit_nX.data.dtype
         self.work_arrays = xp.empty(shape, dtype)
 
@@ -90,20 +89,23 @@ class Davidson(Eigensolver):
 
         assert self.M_nn is not None
 
-        dS = state.ibzwfs.wfs_qs[0][0].setups.overlap_correction
+        wfs = state.ibzwfs.wfs_qs[0][0]
+        dS_aii = wfs.setups.get_overlap_corrections(wfs.P_ani.layout.atomdist,
+                                                    wfs.xp)
         dH = state.potential.dH
-        Ht = partial(hamiltonian.apply, state.potential.vt_sR)
+        Ht = partial(hamiltonian.apply,
+                     state.potential.vt_sR, state.potential.dedtaut_sR)
         ibzwfs = state.ibzwfs
         error = 0.0
 
         weight_un = calculate_weights(self.converge_bands, ibzwfs)
 
-        for wfs, weight_n in zip(ibzwfs, weight_un):
-            e = self.iterate1(wfs, Ht, dH, dS, weight_n)
+        for wfs, weight_n in zips(ibzwfs, weight_un):
+            e = self.iterate1(wfs, Ht, dH, dS_aii, weight_n)
             error += wfs.weight * e
-        return ibzwfs.kpt_comm.sum(float(error)) * ibzwfs.spin_degeneracy
+        return ibzwfs.kpt_band_comm.sum(float(error)) * ibzwfs.spin_degeneracy
 
-    def iterate1(self, wfs, Ht, dH, dS, weight_n):
+    def iterate1(self, wfs, Ht, dH, dS_aii, weight_n):
         H_NN = self.H_NN
         S_NN = self.S_NN
         M_nn = self.M_nn
@@ -111,11 +113,12 @@ class Davidson(Eigensolver):
         xp = M_nn.xp
 
         psit_nX = wfs.psit_nX
-        psit2_nX = psit_nX.new(data=self.work_arrays[0])
-        psit3_nX = psit_nX.new(data=self.work_arrays[1])
-
         B = psit_nX.dims[0]  # number of bands
         eig_N = xp.empty(2 * B)
+        b = psit_nX.mydims[0]
+
+        psit2_nX = psit_nX.new(data=self.work_arrays[0, :b])
+        psit3_nX = psit_nX.new(data=self.work_arrays[1, :b])
 
         wfs.subspace_diagonalize(Ht, dH,
                                  work_array=psit2_nX.data,
@@ -130,8 +133,7 @@ class Davidson(Eigensolver):
         band_comm = psit_nX.comm
         is_domain_band_master = domain_comm.rank == 0 and band_comm.rank == 0
 
-        M0_nn = M_nn
-        assert band_comm.size == 1
+        M0_nn = M_nn.new(dist=(band_comm, 1, 1))
 
         if domain_comm.rank == 0:
             eig_N[:B] = xp.asarray(wfs.eig_n)
@@ -147,7 +149,7 @@ class Davidson(Eigensolver):
         Ht = partial(Ht, out=residual_nX, spin=wfs.spin)
         dH = partial(dH, spin=wfs.spin)
 
-        calculate_residuals(residual_nX, dH, dS, wfs, P2_ani, P3_ani)
+        calculate_residuals(residual_nX, dH, dS_aii, wfs, P2_ani, P3_ani)
 
         def copy(C_nn: Array2D) -> None:
             domain_comm.sum(M_nn.data, 0)
@@ -158,7 +160,7 @@ class Davidson(Eigensolver):
 
         for i in range(self.niter):
             if i == self.niter - 1:  # last iteration
-                # Calulate error before we destroy residuals:
+                # Calculate error before we destroy residuals:
                 if weight_n is None:
                     error = np.inf
                 else:
@@ -185,7 +187,7 @@ class Davidson(Eigensolver):
 
             # <psi2 | S | psi2>
             me(psit2_nX, psit2_nX)
-            dS(P2_ani, out_ani=P3_ani)
+            P2_ani.block_diag_multiply(dS_aii, out_ani=P3_ani)
             P2_ani.matrix.multiply(P3_ani, opb='C', symmetric=True, beta=1,
                                    out=M_nn)
             copy(S_NN.data[B:, B:])
@@ -229,38 +231,40 @@ class Davidson(Eigensolver):
 
             if i < self.niter - 1:
                 Ht(psit_nX)
-                calculate_residuals(residual_nX, dH, dS, wfs, P2_ani, P3_ani)
+                calculate_residuals(
+                    residual_nX, dH, dS_aii, wfs, P2_ani, P3_ani)
 
         return error
 
 
-def calculate_residuals(residual_nX: DA,
-                        dH: AAFunc,
-                        dS: AAFunc,
+def calculate_residuals(residual_nX: XArray,
+                        dH: Callable[[AtomArrays, AtomArrays], AtomArrays],
+                        dS_aii: AtomArrays,
                         wfs: PWFDWaveFunctions,
-                        P1_ani: AA,
-                        P2_ani: AA) -> None:
+                        P1_ani: AtomArrays,
+                        P2_ani: AtomArrays) -> None:
 
     eig_n = wfs.myeig_n
     xp = residual_nX.xp
     if xp is np:
-        for r, e, p in zip(residual_nX.data, wfs.myeig_n, wfs.psit_nX.data):
+        for r, e, p in zips(residual_nX.data, wfs.myeig_n, wfs.psit_nX.data):
             axpy(-e, p, r)
     else:
         eig_n = xp.asarray(eig_n)
-        for r, e, p in zip(residual_nX.data, eig_n, wfs.psit_nX.data):
+        for r, e, p in zips(residual_nX.data, eig_n, wfs.psit_nX.data):
             r -= p * e
 
     dH(wfs.P_ani, P1_ani)
+    wfs.P_ani.block_diag_multiply(dS_aii, out_ani=P2_ani)
+
     if wfs.ncomponents < 4:
         subscripts = 'nI, n -> nI'
     else:
         subscripts = 'nsI, n -> nsI'
     if xp is np:
-        np.einsum(subscripts, wfs.P_ani.data, eig_n, out=P2_ani.data)
+        np.einsum(subscripts, P2_ani.data, eig_n, out=P2_ani.data)
     else:
-        P2_ani.data[:] = xp.einsum(subscripts, wfs.P_ani.data, eig_n)
-    dS(P2_ani, P2_ani)
+        P2_ani.data[:] = xp.einsum(subscripts, P2_ani.data, eig_n)
     P1_ani.data -= P2_ani.data
     wfs.pt_aiX.add_to(residual_nX, P1_ani)
 
@@ -268,7 +272,6 @@ def calculate_residuals(residual_nX: DA,
 def calculate_weights(converge_bands: int | str,
                       ibzwfs: IBZWaveFunctions) -> list[Array1D | None]:
     """Calculate convergence weights for all eigenstates."""
-    assert ibzwfs.band_comm.size == 1, 'not implemented!'
     weight_un = []
     nu = len(ibzwfs.wfs_qs) * ibzwfs.nspins
     nbands = ibzwfs.nbands
@@ -279,7 +282,7 @@ def calculate_weights(converge_bands: int | str,
             try:
                 # Methfessel-Paxton or cold-smearing distributions can give
                 # negative occupation numbers - so we take the absolute value:
-                weight_n = np.abs(wfs.occ_n)
+                weight_n = np.abs(wfs.myocc_n)
             except ValueError:
                 # No eigenvalues yet:
                 return [None] * nu
@@ -287,20 +290,23 @@ def calculate_weights(converge_bands: int | str,
         return weight_un
 
     if converge_bands == 'all':
-        return [np.ones(nbands)] * nu
+        converge_bands = nbands
 
     if isinstance(converge_bands, int):
         # Converge fixed number of bands:
         n = converge_bands
-
+        if n < 0:
+            n += nbands
+            assert n >= 0
         for wfs in ibzwfs:
-            weight_n = np.zeros(nbands)
-            if n < 0:
-                n += nbands
-            weight_n[:n] = 1.0
+            weight_n = np.zeros(wfs.n2 - wfs.n1)
+            m = max(wfs.n1, min(n, wfs.n2)) - wfs.n1
+            weight_n[:m] = 1.0
             weight_un.append(weight_n)
         return weight_un
 
+    assert ibzwfs.band_comm.size == 1, 'not implemented!'
+
     # Converge states with energy up to CBM + delta:
     assert converge_bands.startswith('CBM+')
     delta = float(converge_bands[4:]) / Ha
