@@ -1,34 +1,13 @@
-from math import factorial as fac
 from math import pi
 
 import _gpaw
 import numpy as np
-from gpaw.lcao.overlap import fbt
 from gpaw.lfc import BaseLFC
 from gpaw.spherical_harmonics import Y, nablarlYL
-from gpaw.spline import Spline
+from gpaw.ffbt import rescaled_fourier_bessel_transform
 from gpaw.utilities.blas import mmm
 
 
-def ft(spline, N=2**10):
-    l = spline.get_angular_momentum_number()
-    rc = 50.0
-    assert spline.get_cutoff() <= rc
-
-    dr = rc / N
-    r_r = np.arange(N) * dr
-    dk = pi / 2 / rc
-    k_q = np.arange(2 * N) * dk
-    f_r = spline.map(r_r) * (4 * pi)
-
-    f_q = fbt(l, f_r, r_r, k_q)
-    f_q[1:] /= k_q[1:]**(2 * l + 1)
-    f_q[0] = (np.dot(f_r, r_r**(2 + 2 * l)) *
-              dr * 2**l * fac(l) / fac(2 * l + 1))
-
-    return Spline(l, k_q[-1], f_q)
-
-
 class PWLFC(BaseLFC):
     def __init__(self, spline_aj, pd, blocksize=5000, comm=None):
         """Reciprocal-space plane-wave localized function collection.
@@ -105,7 +84,7 @@ class PWLFC(BaseLFC):
             for spline in spline_j:
                 s = splines[spline]  # get spline index
                 if spline not in done:
-                    f = ft(spline)
+                    f = rescaled_fourier_bessel_transform(spline)
                     for f_Gs, G2_G in zip(self.f_qGs, self.pd.G2_qG):
                         G_G = G2_G**0.5
                         f_Gs[:, s] = f.map(G_G)
@@ -377,7 +356,7 @@ class PWLFC(BaseLFC):
         for a, spline_j in enumerate(self.spline_aj):
             for spline in spline_j:
                 if spline not in cache:
-                    s = ft(spline)
+                    s = rescaled_fourier_bessel_transform(spline)
                     G_G = self.pd.G2_qG[q]**0.5
                     f_G = []
                     dfdGoG_G = []
