@@ -1,4 +1,3 @@
-import functools
 from time import time, ctime
 from datetime import timedelta
 
@@ -16,19 +15,18 @@ from gpaw.response.coulomb_kernels import CoulombKernel
 from gpaw.response.screened_interaction import initialize_w_calculator
 from gpaw.response.paw import PWPAWCorrectionData
 from gpaw.response.frequencies import FrequencyDescriptor
-from gpaw.response.pair import PairDensityCalculator, get_gs_and_context
+from gpaw.response.pair import KPointPairFactory, get_gs_and_context
 from gpaw.response.pair_functions import SingleQPWDescriptor
 from gpaw.response.chi0 import Chi0Calculator
 
 
 class BSEBackend:
     def __init__(self, *, gs, context,
+                 valence_bands, conduction_bands,
                  spinors=False,
                  ecut=10.,
                  scale=1.0,
                  nbands=None,
-                 valence_bands=None,
-                 conduction_bands=None,
                  eshift=None,
                  gw_skn=None,
                  truncation=None,
@@ -77,20 +75,10 @@ class BSEBackend:
                                    'version does not work for spin-polarized' +
                                    ' calculations. Performing scalar ' +
                                    'calculation')
-            assert len(valence_bands[0]) == len(valence_bands[1])
-            assert len(conduction_bands[0]) == len(conduction_bands[1])
-        if valence_bands is None:
-            nv = self.gs.nvalence
-            valence_bands = [[nv // 2 - 1]]
-            if self.spins == 2:
-                valence_bands *= 2
-        if conduction_bands is None:
-            conduction_bands = [[valence_bands[-1] + 1]]
-            if self.spins == 2:
-                conduction_bands *= 2
 
-        self.val_sn = np.atleast_2d(valence_bands)
-        self.con_sn = np.atleast_2d(conduction_bands)
+        self.val_sn = self.parse_bands(valence_bands, band_type='valence')
+        self.con_sn = self.parse_bands(conduction_bands,
+                                       band_type='conduction')
 
         self.td = True
         for n in self.val_sn[0]:
@@ -118,7 +106,7 @@ class BSEBackend:
         self.nS = self.kd.nbzkpts * self.nv * self.nc * self.spins
         self.nS *= (self.spinors + 1)**2
 
-        self.coulomb = CoulombKernel(truncation=truncation, gs=self.gs)
+        self.coulomb = CoulombKernel.from_gs(self.gs, truncation=truncation)
         self.context.print(self.coulomb.description())
 
         self.print_initialization(self.td, self.eshift, self.gw_skn)
@@ -127,6 +115,65 @@ class BSEBackend:
         self._chi0calc = None  # Initialized later
         self._wcalc = None  # Initialized later
 
+    @property
+    def pair_calc(self):
+        return self.kptpair_factory.pair_calculator()
+
+    def parse_bands(self, bands, band_type='valence'):
+        """Helper function that checks whether bands are correctly specified,
+         and brings them to the format used later in the code.
+
+        If the calculation is spin-polarized, band indices must
+        be given explicitly as lists/arrays of shape (2,nbands) where the first
+        index is for spin.
+
+        If the calculation is not spin-polarized, either an integer (number of
+        desired bands) or lists of band indices must be provided.
+
+        band_type is an optional parameter that is only when a desired number
+        of bands is given (rather than a list) to help figure out the correct
+        band indices.
+        """
+        if hasattr(bands, '__iter__'):
+            if self.spins == 2:
+                if len(bands) != 2 or (len(bands[0]) != len(bands[1])):
+                    raise ValueError('For a spin-polarized calculation, '
+                                     'the same number of bands must be '
+                                     'specified for each spin! valence and '
+                                     'conduction bands must be lists of shape '
+                                     '(2,n)')
+
+            bands_sn = np.atleast_2d(bands)
+            return bands_sn
+
+        # if we get here, bands is not iterable
+        # check that the specified input is valid
+
+        if self.spins == 2:
+            raise NotImplementedError('For a spin-polarized calculation, '
+                                      'bands must be specified as lists '
+                                      'of shape (2,n)')
+
+        n_fully_occupied_bands, n_partially_occupied_bands = \
+            self.gs.count_occupied_bands()
+
+        if n_fully_occupied_bands != n_partially_occupied_bands:
+            raise NotImplementedError('Automatic band generation is currently '
+                                      'not implemented for metallic systems. '
+                                      'Please specify band indices manually.')
+
+        if band_type == 'valence':
+            bands_sn = range(n_fully_occupied_bands - bands,
+                             n_fully_occupied_bands)
+        elif band_type == 'conduction':
+            bands_sn = range(n_fully_occupied_bands,
+                             n_fully_occupied_bands + bands)
+        else:
+            raise ValueError(f'Invalid band type: {band_type}')
+
+        bands_sn = np.atleast_2d(bands_sn)
+        return bands_sn
+
     def calculate(self, optical=True):
 
         if self.spinors:
@@ -153,7 +200,7 @@ class BSEBackend:
         if optical:
             v_G[0] = 0.0
 
-        self.pair = PairDensityCalculator(
+        self.kptpair_factory = KPointPairFactory(
             gs=self.gs,
             context=ResponseContext(txt='pair.txt', timer=None,
                                     comm=serial_comm))
@@ -183,8 +230,8 @@ class BSEBackend:
 
         optical_limit = np.allclose(self.q_c, 0.0)
 
-        get_pair = self.pair.get_kpoint_pair
-        get_pair_density = self.pair.get_pair_density
+        get_pair = self.kptpair_factory.get_kpoint_pair
+        get_pair_density = self.pair_calc.get_pair_density
         if self.spinors:
             # Get all pair densities to allow for SOC mixing
             # Use twice as many no-SOC states as BSE bands to allow mixing
@@ -225,8 +272,8 @@ class BSEBackend:
                 rho_mnG = get_pair_density(qpd0, pair, m_m, n_n,
                                            pawcorr=pawcorr)
                 if optical_limit:
-                    n_mnv = self.pair.get_optical_pair_density_head(qpd0, pair,
-                                                                    m_m, n_n)
+                    n_mnv = self.pair_calc.get_optical_pair_density_head(
+                        qpd0, pair, m_m, n_n)
                     rho_mnG[:, :, 0] = n_mnv[:, :, self.direction]
                 if self.spinors:
                     v0_kmn = v_kmn[:, :, ::2]
@@ -272,9 +319,10 @@ class BSEBackend:
         H_ksmnKsmn = np.zeros((myKsize, Ns, Nv, Nc, nK, Ns, Nv, Nc), complex)
         for ik1, iK1 in enumerate(myKrange):
             for s1 in range(Ns):
-                kptv1 = self.pair.get_k_point(s1, iK1, vi_s[s1], vf_s[s1])
-                kptc1 = self.pair.get_k_point(s1, ikq_k[iK1], ci_s[s1],
-                                              cf_s[s1])
+                kptv1 = self.kptpair_factory.get_k_point(
+                    s1, iK1, vi_s[s1], vf_s[s1])
+                kptc1 = self.kptpair_factory.get_k_point(
+                    s1, ikq_k[iK1], ci_s[s1], cf_s[s1])
                 rho1_mnG = rhoex_KsmnG[iK1, s1]
 
                 # rhoG0_Ksmn[iK1, s1] = rho1_mnG[:, :, 0]
@@ -290,10 +338,10 @@ class BSEBackend:
 
                         if not self.mode == 'RPA' and s1 == s2:
                             ikq = ikq_k[iK2]
-                            kptv2 = self.pair.get_k_point(s1, iK2, vi_s[s1],
-                                                          vf_s[s1])
-                            kptc2 = self.pair.get_k_point(s1, ikq, ci_s[s1],
-                                                          cf_s[s1])
+                            kptv2 = self.kptpair_factory.get_k_point(
+                                s1, iK2, vi_s[s1], vf_s[s1])
+                            kptc2 = self.kptpair_factory.get_k_point(
+                                s1, ikq, ci_s[s1], cf_s[s1])
                             rho3_mmG, iq = self.get_density_matrix(kptv1,
                                                                    kptv2)
                             rho4_nnG, iq = self.get_density_matrix(kptc1,
@@ -361,7 +409,7 @@ class BSEBackend:
 
         if self.write_h:
             self.par_save('H_SS.ulm', 'H_SS', self.H_sS)
-    
+
     def get_density_matrix(self, kpt1, kpt2):
         from gpaw.response.g0w0 import QSymmetryOp, get_nmG
         symop, iq = QSymmetryOp.get_symop_from_kpair(self.kd, self.qd,
@@ -373,7 +421,8 @@ class BSEBackend:
         rho_mnG = np.zeros((len(kpt1.eps_n), len(kpt2.eps_n), nG),
                            complex)
         for m in range(len(rho_mnG)):
-            rho_mnG[m] = get_nmG(kpt1, kpt2, pawcorr, m, qpd, I_G, self.pair)
+            rho_mnG[m] = get_nmG(kpt1, kpt2, pawcorr, m, qpd, I_G,
+                                 self.pair_calc)
         return rho_mnG, iq
 
     def get_screened_potential(self):
@@ -391,7 +440,8 @@ class BSEBackend:
                     PWPAWCorrectionData(
                         Q_aGii, qpd=qpd,
                         pawdatasets=self.gs.pawdatasets,
-                        pos_av=self.gs.get_pos_av())
+                        pos_av=self.gs.get_pos_av(),
+                        atomrotations=self.gs.atomrotations)
                     for Q_aGii, qpd in zip(data['Q'], self.qpd_q)]
                 self.W_qGG = data['W']
                 self.context.print('Reading screened potential from % s' %
@@ -412,20 +462,20 @@ class BSEBackend:
         susceptibility."""
 
         wd = FrequencyDescriptor([0.0])
-        pair = PairDensityCalculator(
+        kptpair_factory = KPointPairFactory(
             gs=self.gs,
             context=self.context.with_txt('chi0.txt'))
 
         self._chi0calc = Chi0Calculator(
             wd=wd,
-            pair=pair,
+            kptpair_factory=kptpair_factory,
             eta=0.001,
             ecut=self.ecut * Hartree,
             intraband=False,
             hilbert=False,
             nbands=self.nbands)
 
-        self.blockcomm = self._chi0calc.blockcomm
+        self.blockcomm = self._chi0calc.integrator.blockcomm
 
     def calculate_screened_potential(self):
         """Calculate W_GG(q)"""
@@ -449,7 +499,10 @@ class BSEBackend:
             chi0 = self._chi0calc.calculate(q_c)
             W_wGG = self._wcalc.calculate_W_wGG(chi0)
             W_GG = W_wGG[0]
-            self.pawcorr_q.append(self._chi0calc.pawcorr)
+            # This is such a terrible way to access the paw
+            # corrections. Attributes should not be groped like
+            # this... Change in the future! XXX
+            self.pawcorr_q.append(self._chi0calc.chi0_body_calc.pawcorr)
             self.qpd_q.append(chi0.qpd)
             self.W_qGG.append(W_GG)
 
@@ -835,61 +888,65 @@ class BSEBackend:
         return myKrange, myKsize, mySsize
 
     def print_initialization(self, td, eshift, gw_skn):
-        p = functools.partial(self.context.print, flush=False)
-        p('----------------------------------------------------------')
-        p('%s Hamiltonian' % self.mode)
-        p('----------------------------------------------------------')
-        p('Started at:  ', ctime())
-        p()
-        p('Atoms                          :',
-          self.gs.atoms.get_chemical_formula(mode='hill'))
-        p('Ground state XC functional     :', self.gs.xcname)
-        p('Valence electrons              :', self.gs.nvalence)
-        p('Spinor calculations            :', self.spinors)
-        p('Number of bands                :', self.gs.bd.nbands)
-        p('Number of spins                :', self.gs.nspins)
-        p('Number of k-points             :', self.kd.nbzkpts)
-        p('Number of irreducible k-points :', self.kd.nibzkpts)
-        p('Number of q-points             :', self.qd.nbzkpts)
-        p('Number of irreducible q-points :', self.qd.nibzkpts)
-        p()
+        isl = ['----------------------------------------------------------',
+               f'{self.mode} Hamiltonian',
+               '----------------------------------------------------------',
+               f'Started at:  {ctime()}', '',
+               'Atoms                          : '
+               f'{self.gs.atoms.get_chemical_formula(mode="hill")}',
+               f'Ground state XC functional     : {self.gs.xcname}',
+               f'Valence electrons              : {self.gs.nvalence}',
+               f'Spinor calculations            : {self.spinors}',
+               f'Number of bands                : {self.gs.bd.nbands}',
+               f'Number of spins                : {self.gs.nspins}',
+               f'Number of k-points             : {self.kd.nbzkpts}',
+               f'Number of irreducible k-points : {self.kd.nibzkpts}',
+               f'Number of q-points             : {self.qd.nbzkpts}',
+               f'Number of irreducible q-points : {self.qd.nibzkpts}', '']
+
         for q in self.qd.ibzk_kc:
-            p('    q: [%1.4f %1.4f %1.4f]' % (q[0], q[1], q[2]))
-        p()
+            isl.append(f'    q: [{q[0]:1.4f} {q[1]:1.4f} {q[2]:1.4f}]')
+        isl.append('')
         if gw_skn is not None:
-            p('User specified BSE bands')
-        p('Response PW cutoff             :', self.ecut * Hartree, 'eV')
-        p('Screening bands included       :', self.nbands)
+            isl.append('User specified BSE bands')
+        isl.extend([f'Response PW cutoff             : {self.ecut * Hartree} '
+                    f'eV',
+                    f'Screening bands included       : {self.nbands}'])
         if len(self.val_sn) == 1:
-            p('Valence bands                  :', self.val_sn[0])
-            p('Conduction bands               :', self.con_sn[0])
+            isl.extend([f'Valence bands                  : {self.val_sn[0]}',
+                        f'Conduction bands               : {self.con_sn[0]}'])
         else:
-            p('Valence bands                  :', self.val_sn[0],
-              self.val_sn[1])
-            p('Conduction bands               :', self.con_sn[0],
-              self.con_sn[1])
+            isl.extend([f'Valence bands                  : {self.val_sn[0]}'
+                        f' {self.val_sn[1]}',
+                        f'Conduction bands               : {self.con_sn[0]}'
+                        f' {self.con_sn[1]}'])
         if eshift is not None:
-            p('Scissors operator              :', eshift * Hartree, 'eV')
-        p('Tamm-Dancoff approximation     :', td)
-        p('Number of pair orbitals        :', self.nS)
-        p()
-        p('Truncation of Coulomb kernel   :', self.coulomb.truncation)
+            isl.append(f'Scissors operator              : {eshift * Hartree}'
+                       f'eV')
+        isl.extend([
+            f'Tamm-Dancoff approximation     : {td}',
+            f'Number of pair orbitals        : {self.nS}',
+            '',
+            f'Truncation of Coulomb kernel   : {self.coulomb.truncation}'])
         if self.integrate_gamma == 0:
-            p('Coulomb integration scheme     :', 'Analytical - gamma only')
+            isl.append(
+                'Coulomb integration scheme     : Analytical - gamma only')
         elif self.integrate_gamma == 1:
-            p('Coulomb integration scheme     :', 'Numerical - all q-points')
+            isl.append(
+                'Coulomb integration scheme     : Numerical - all q-points')
         else:
             pass
-        p()
-        p('----------------------------------------------------------')
-        p('----------------------------------------------------------')
-        p()
-        p('Parallelization - Total number of CPUs   : % s' % world.size)
-        p('  Screened potential')
-        p('    K-point/band decomposition           : % s' % world.size)
-        p('  Hamiltonian')
-        p('    Pair orbital decomposition           : % s' % world.size)
-        self.context.print('')
+        isl.extend([
+            '',
+            '----------------------------------------------------------',
+            '----------------------------------------------------------',
+            '',
+            f'Parallelization - Total number of CPUs   : {world.size}',
+            '  Screened potential',
+            f'    K-point/band decomposition           : {world.size}',
+            '  Hamiltonian',
+            f'    Pair orbital decomposition           : {world.size}'])
+        self.context.print('\n'.join(isl))
 
 
 class BSE(BSEBackend):
