@@ -9,7 +9,7 @@ import gpaw.mpi as mpi
 from gpaw.response.coulomb_kernels import CoulombKernel
 from gpaw.response.density_kernels import get_density_xc_kernel
 from gpaw.response.chi0 import Chi0Calculator, new_frequency_descriptor
-from gpaw.response.pair import get_gs_and_context, PairDensityCalculator
+from gpaw.response.pair import get_gs_and_context, KPointPairFactory
 
 
 class DielectricFunctionCalculator:
@@ -17,7 +17,7 @@ class DielectricFunctionCalculator:
         from gpaw.response.pw_parallelization import Blocks1D
         self.chi0calc = chi0calc
 
-        self.coulomb = CoulombKernel(truncation=truncation, gs=self.gs)
+        self.coulomb = CoulombKernel.from_gs(self.gs, truncation=truncation)
         self.context = chi0calc.context
         self.wd = chi0calc.wd
         self.blocks1d = Blocks1D(self.context.comm, len(self.wd))
@@ -73,7 +73,7 @@ class DielectricFunctionCalculator:
             self._chi0cache.clear()
 
             chi0 = self.chi0calc.calculate(q_c, spin)
-            chi0_wGG = chi0.get_distributed_frequencies_array()
+            chi0_wGG = chi0.body.get_distributed_frequencies_array()
             self.context.write_timer()
             things = chi0.qpd, chi0_wGG, chi0.chi0_WxvG, chi0.chi0_Wvv
             self._chi0cache[key] = things
@@ -115,7 +115,7 @@ class DielectricFunctionCalculator:
         """
         qpd, chi0_wGG, chi0_WxvG, chi0_Wvv = self.calculate_chi0(q_c, spin)
 
-        coulomb_bare = CoulombKernel(truncation=None, gs=self.gs)
+        coulomb_bare = CoulombKernel.from_gs(self.gs, truncation=None)
         Kbare_G = coulomb_bare.V(qpd=qpd, q_v=q_v)
         sqrtV_G = Kbare_G**0.5
 
@@ -495,7 +495,7 @@ class DielectricFunction(DielectricFunctionCalculator):
                  omegamax=None,  # deprecated
                  ecut=50,
                  hilbert=True,
-                 nbands=None, eta=0.2, threshold=1,
+                 nbands=None, eta=0.2,
                  intraband=True, nblocks=1, world=mpi.world, txt=sys.stdout,
                  truncation=None, disable_point_group=False,
                  disable_time_reversal=False,
@@ -519,9 +519,6 @@ class DielectricFunction(DielectricFunctionCalculator):
             Number of bands from calculation.
         eta: float
             Broadening parameter.
-        threshold: float
-            Threshold for matrix elements in optical response perturbation
-            theory.
         intraband: bool
             Include intraband transitions.
         world: comm
@@ -546,12 +543,12 @@ class DielectricFunction(DielectricFunctionCalculator):
                                       domega0=domega0,
                                       omega2=omega2, omegamax=omegamax)
 
-        pair = PairDensityCalculator(
-            gs=gs, context=context, threshold=threshold, nblocks=nblocks)
+        kptpair_factory = KPointPairFactory(
+            gs=gs, context=context, nblocks=nblocks)
 
         chi0calc = Chi0Calculator(
             wd=wd,
-            pair=pair,
+            kptpair_factory=kptpair_factory,
             ecut=ecut, nbands=nbands, eta=eta,
             hilbert=hilbert,
             intraband=intraband,
