@@ -32,7 +32,7 @@ def czher(alpha: float, x, A) -> None:
 
 
 class Integrator:
-    def __init__(self, cell_cv, context, nblocks=1, eshift=0.0):
+    def __init__(self, cell_cv, context, *, nblocks):
         """Baseclass for Brillouin zone integration and band summation.
 
         Simple class to calculate integrals over Brilloun zones
@@ -43,8 +43,6 @@ class Integrator:
         """
 
         self.context = context
-        self.eshift = eshift
-        self.nblocks = nblocks
         self.vol = abs(np.linalg.det(cell_cv))
 
         self.blockcomm, self.kncomm = block_partition(self.context.comm,
@@ -78,12 +76,9 @@ class Integrator:
 
         return mydomain
 
-    def integrate(self, *args, **kwargs):
+    def integrate(self, **kwargs):
         raise NotImplementedError
 
-    def _blocks1d(self, nG):
-        return Blocks1D(self.blockcomm, nG)
-
 
 class PointIntegrator(Integrator):
     """Integrate brillouin zone using a broadening technique.
@@ -92,48 +87,11 @@ class PointIntegrator(Integrator):
     delta functions appearing in many integrals by some factor
     eta. In this code we use Lorentzians."""
 
-    def integrate(self, kind='pointwise', *args, **kwargs):
-        self.context.print('Integral kind:', kind)
-        if kind == 'pointwise':
-            return self.pointwise_integration(*args, **kwargs)
-        elif kind == 'hermitian response function':
-            return self.response_function_integration(hermitian=True,
-                                                      hilbert=False,
-                                                      wings=False,
-                                                      *args, **kwargs)
-        elif kind == 'hermitian response function wings':
-            return self.response_function_integration(hermitian=True,
-                                                      hilbert=False,
-                                                      wings=True,
-                                                      *args, **kwargs)
-        elif kind == 'spectral function':
-            return self.response_function_integration(hilbert=True,
-                                                      *args, **kwargs)
-        elif kind == 'spectral function wings':
-            return self.response_function_integration(hilbert=True,
-                                                      wings=True,
-                                                      *args, **kwargs)
-        elif kind == 'response function':
-            return self.response_function_integration(hilbert=False,
-                                                      *args, **kwargs)
-        elif kind == 'response function wings':
-            return self.response_function_integration(hilbert=False,
-                                                      wings=True,
-                                                      *args, **kwargs)
-        else:
-            raise ValueError(kind)
+    def integrate(self, *, task, wd, domain, integrand, out_wxx):
+        """Integrate a response function over bands and kpoints."""
 
-    def response_function_integration(self, *, domain, integrand,
-                                      x=None, out_wxx,
-                                      hermitian=False,
-                                      intraband=False, hilbert=False,
-                                      wings=False, eta=None):
-        """Integrate a response function over bands and kpoints.
+        self.context.print('Integral kind:', task.kind)
 
-        func: method
-        omega_w: ndarray
-        out: np.ndarray
-        """
         mydomain_t = self.distribute_domain(domain)
         nbz = len(domain[0])
 
@@ -149,27 +107,7 @@ class PointIntegrator(Integrator):
                 continue
             deps_M = integrand.eigenvalues(*arguments)
 
-            if intraband:
-                assert eta is None
-                assert x is None
-                self.update_intraband(n_MG, out_wxx)
-            elif hermitian and not wings:
-                assert eta is None
-                self.update_hermitian(n_MG, deps_M, x, out_wxx)
-            elif hermitian and wings:
-                assert eta is None
-                self.update_hermitian_optical_limit(n_MG, deps_M, x, out_wxx)
-            elif hilbert and not wings:
-                assert eta is None
-                self.update_hilbert(n_MG, deps_M, x, out_wxx)
-            elif hilbert and wings:
-                assert eta is None
-                self.update_hilbert_optical_limit(n_MG, deps_M, x, out_wxx)
-            elif wings:
-                self.update_optical_limit(n_MG, deps_M, x, out_wxx,
-                                          eta=eta)
-            else:
-                self.update(n_MG, deps_M, x, out_wxx, eta=eta)
+            task.run(wd, n_MG, deps_M, out_wxx)
 
         # Sum over
         # Can this really be valid, if the original input out_wxx is nonzero?
@@ -177,12 +115,15 @@ class PointIntegrator(Integrator):
         # There could also be similar errors elsewhere... XXX
         self.kncomm.sum(out_wxx)
 
-        if (hermitian or hilbert) and self.blockcomm.size == 1 and not wings:
+        if self.blockcomm.size == 1 and task.symmetrizable_unless_blocked:
             # Fill in upper/lower triangle also:
             nx = out_wxx.shape[1]
             il = np.tril_indices(nx, -1)
             iu = il[::-1]
-            if hilbert:
+
+            if isinstance(task, Hilbert):
+                # XXX special hack since one of them wants the other
+                # triangle.
                 for out_xx in out_wxx:
                     out_xx[il] = out_xx[iu].conj()
             else:
@@ -191,19 +132,48 @@ class PointIntegrator(Integrator):
 
         out_wxx *= prefactor
 
-    @timer('CHI_0 update')
-    def update(self, n_mG, deps_m, wd, chi0_wGG, eta):
+
+class IntegralTask(ABC):
+    # Unique string for each kind of integral:
+    kind = '(unset)'
+
+    # Some integrals kinds like to calculate upper or lower half of the output
+    # when nblocks==1.  In that case, this boolean signifies to the
+    # integrator that the output array should be symmetrized.
+    #
+    # Actually: We don't gain anything much by doing this boolean
+    # more systematically, since it's just Hermitian and Hilbert that need
+    # it, and then one of the Tetrahedron types which is not compatible
+    # anyway.  We should probably not do this.
+    symmetrizable_unless_blocked = False
+
+    @abstractmethod
+    def run(self, wd, n_mG, deps_m, out_wxx):
+        """Add contribution from one point to out_wxx."""
+
+
+class GenericUpdate(IntegralTask):
+    kind = 'response function'
+    symmetrizable_unless_blocked = False
+
+    def __init__(self, eta, blockcomm, eshift=0.0):
+        self.eta = eta
+        self.blockcomm = blockcomm
+        self.eshift = eshift
+
+    # @timer('CHI_0 update')
+    def run(self, wd, n_mG, deps_m, chi0_wGG):
         """Update chi."""
 
         deps_m += self.eshift * np.sign(deps_m)
-        deps1_m = deps_m + 1j * eta
-        deps2_m = deps_m - 1j * eta
+        deps1_m = deps_m + 1j * self.eta
+        deps2_m = deps_m - 1j * self.eta
 
-        blocks1d = self._blocks1d(chi0_wGG.shape[2])
+        blocks1d = Blocks1D(self.blockcomm, chi0_wGG.shape[2])
 
         for omega, chi0_GG in zip(wd.omega_w, chi0_wGG):
             x_m = (1 / (omega + deps1_m) - 1 / (omega - deps2_m))
-            if self.blockcomm.size > 1:
+            if blocks1d.blockcomm.size > 1:
                 nx_mG = n_mG[:, blocks1d.myslice] * x_m[:, np.newaxis]
             else:
                 nx_mG = n_mG * x_m[:, np.newaxis]
@@ -211,15 +181,24 @@ class PointIntegrator(Integrator):
             mmm(1.0, np.ascontiguousarray(nx_mG.T), 'N', n_mG.conj(), 'N',
                 1.0, chi0_GG)
 
-    @timer('CHI_0 hermetian update')
-    def update_hermitian(self, n_mG, deps_m, wd, chi0_wGG):
+
+class Hermitian(IntegralTask):
+    kind = 'hermitian response function'
+    symmetrizable_unless_blocked = True
+
+    def __init__(self, blockcomm, eshift=0.0):
+        self.blockcomm = blockcomm
+        self.eshift = eshift
+
+    # @timer('CHI_0 hermetian update')
+    def run(self, wd, n_mG, deps_m, chi0_wGG):
         """If eta=0 use hermitian update."""
         deps_m += self.eshift * np.sign(deps_m)
 
-        blocks1d = self._blocks1d(chi0_wGG.shape[2])
+        blocks1d = Blocks1D(self.blockcomm, chi0_wGG.shape[2])
 
         for w, omega in enumerate(wd.omega_w):
-            if self.blockcomm.size == 1:
+            if blocks1d.blockcomm.size == 1:
                 x_m = np.abs(2 * deps_m / (omega.imag**2 + deps_m**2))**0.5
                 nx_mG = n_mG.conj() * x_m[:, np.newaxis]
                 rk(-1.0, nx_mG, 1.0, chi0_wGG[w], 'n')
@@ -228,8 +207,17 @@ class PointIntegrator(Integrator):
                 mynx_mG = n_mG[:, blocks1d.myslice] * x_m[:, np.newaxis]
                 mmm(-1.0, mynx_mG, 'T', n_mG.conj(), 'N', 1.0, chi0_wGG[w])
 
-    @timer('CHI_0 spectral function update (new)')
-    def update_hilbert(self, n_mG, deps_m, wd, chi0_wGG):
+
+class Hilbert(IntegralTask):
+    kind = 'spectral function'
+    symmetrizable_unless_blocked = True
+
+    def __init__(self, blockcomm, eshift=0.0):
+        self.blockcomm = blockcomm
+        self.eshift = eshift
+
+    # @timer('CHI_0 spectral function update (new)')
+    def run(self, wd, n_mG, deps_m, chi0_wGG):
         """Update spectral function.
 
         Updates spectral function A_wGG and saves it to chi0_wGG for
@@ -239,7 +227,7 @@ class PointIntegrator(Integrator):
         o_m = abs(deps_m)
         w_m = wd.get_floor_index(o_m)
 
-        blocks1d = self._blocks1d(chi0_wGG.shape[2])
+        blocks1d = Blocks1D(self.blockcomm, chi0_wGG.shape[2])
 
         # Sort frequencies
         argsw_m = np.argsort(w_m)
@@ -271,7 +259,7 @@ class PointIntegrator(Integrator):
             p1_m = np.array(p * (o2 - sortedo_m[startindex:endindex]))
             p2_m = np.array(p * (sortedo_m[startindex:endindex] - o1))
 
-            if self.blockcomm.size > 1 and w + 1 < wd.wmax:
+            if blocks1d.blockcomm.size > 1 and w + 1 < wd.wmax:
                 x_mG = sortedn_mG[startindex:endindex, blocks1d.myslice]
                 mmm(1.0,
                     np.concatenate((p1_m[:, None] * x_mG,
@@ -284,7 +272,7 @@ class PointIntegrator(Integrator):
                     chi0_wGG[w:w + 2].reshape((2 * blocks1d.nlocal,
                                                blocks1d.N)))
 
-            if self.blockcomm.size <= 1 and w + 1 < wd.wmax:
+            if blocks1d.blockcomm.size <= 1 and w + 1 < wd.wmax:
                 x_mG = sortedn_mG[startindex:endindex]
                 l_Gm = (p1_m[:, None] * x_mG).T.copy()
                 r_Gm = x_mG.T.copy()
@@ -292,35 +280,59 @@ class PointIntegrator(Integrator):
                 l_Gm = (p2_m[:, None] * x_mG).T.copy()
                 mmm(1.0, r_Gm, 'N', l_Gm, 'C', 1.0, chi0_wGG[w + 1])
 
-    @timer('CHI_0 intraband update')
-    def update_intraband(self, vel_mv, chi0_wvv):
+
+class Intraband(IntegralTask):
+    kind = 'intraband'
+    symmetrizable_unless_blocked = False
+
+    # @timer('CHI_0 intraband update')
+    def run(self, wd, vel_mv, deps_M, chi0_wvv):
         """Add intraband contributions"""
+        # Intraband is a little bit special, we use neither wd nor deps_M
 
         for vel_v in vel_mv:
             x_vv = np.outer(vel_v, vel_v)
             chi0_wvv[0] += x_vv
 
-    @timer('CHI_0 optical limit update')
-    def update_optical_limit(self, n_mG, deps_m, wd, chi0_wxvG, eta):
+
+class OpticalLimit(IntegralTask):
+    kind = 'response function wings'
+    symmetrizable_unless_blocked = False
+
+    def __init__(self, eta):
+        self.eta = eta
+
+    # @timer('CHI_0 optical limit update')
+    def run(self, wd, n_mG, deps_m, chi0_wxvG):
         """Optical limit update of chi."""
-        deps1_m = deps_m + 1j * eta
-        deps2_m = deps_m - 1j * eta
+        deps1_m = deps_m + 1j * self.eta
+        deps2_m = deps_m - 1j * self.eta
 
         for w, omega in enumerate(wd.omega_w):
             x_m = (1 / (omega + deps1_m) - 1 / (omega - deps2_m))
             chi0_wxvG[w, 0] += np.dot(x_m * n_mG[:, :3].T, n_mG.conj())
             chi0_wxvG[w, 1] += np.dot(x_m * n_mG[:, :3].T.conj(), n_mG)
 
-    @timer('CHI_0 hermitian optical limit update')
-    def update_hermitian_optical_limit(self, n_mG, deps_m, wd, chi0_wxvG):
+
+class HermitianOpticalLimit(IntegralTask):
+    kind = 'hermitian response function wings'
+    symmetrizable_unless_blocked = False
+
+    # @timer('CHI_0 hermitian optical limit update')
+    def run(self, wd, n_mG, deps_m, chi0_wxvG):
         """Optical limit update of hermitian chi."""
         for w, omega in enumerate(wd.omega_w):
             x_m = - np.abs(2 * deps_m / (omega.imag**2 + deps_m**2))
             chi0_wxvG[w, 0] += np.dot(x_m * n_mG[:, :3].T, n_mG.conj())
             chi0_wxvG[w, 1] += np.dot(x_m * n_mG[:, :3].T.conj(), n_mG)
 
-    @timer('CHI_0 optical limit hilbert-update')
-    def update_hilbert_optical_limit(self, n_mG, deps_m, wd, chi0_wxvG):
+
+class HilbertOpticalLimit(IntegralTask):
+    kind = 'spectral function wings'
+    symmetrizable_unless_blocked = False
+
+    # @timer('CHI_0 optical limit hilbert-update')
+    def run(self, wd, n_mG, deps_m, chi0_wxvG):
         """Optical limit update of chi-head and -wings."""
 
         for deps, n_G in zip(deps_m, n_mG):
@@ -374,24 +386,8 @@ class TetrahedronIntegrator(Integrator):
 
         return self.get_simplex_volume(td, S)
 
-    def integrate(self, kind, *args, **kwargs):
-        if kind == 'spectral function':
-            wings = False
-        elif kind == 'spectral function wings':
-            wings = True
-        else:
-            raise ValueError("Expected kind='spectral function'",
-                             "or 'spectral function wings', got: ",
-                             kind)
-
-        return self.spectral_function_integration(*args,
-                                                  wings=wings,
-                                                  **kwargs)
-
     @timer('Spectral function integration')
-    def spectral_function_integration(self, wings=False,
-                                      *, domain, integrand,
-                                      x, out_wxx):
+    def integrate(self, *, domain, integrand, wd, out_wxx, task):
         """Integrate response function.
 
         Assume that the integral has the
@@ -399,10 +395,6 @@ class TetrahedronIntegrator(Integrator):
         method it is possible calculate frequency dependent weights
         and do a point summation using these weights."""
 
-        wd = x  # XXX Rename.  But it clashes with some other methods
-        # that are **kwargs'ed somewhere, so requires attention.
-        blocks1d = self._blocks1d(out_wxx.shape[2])
-
         # Input domain
         td = self.tesselate(domain[0])
         args = domain[1:]
@@ -486,16 +478,11 @@ class TetrahedronIntegrator(Integrator):
                                              td)
                 W_Mw.append(W_w)
 
-            if wings:
-                self.update_hilbert_optical_limit(n_MG, deps_Mk, W_Mw,
-                                                  i0_M, i1_M, out_wxx)
-            else:
-                self.update_hilbert(n_MG, deps_Mk, W_Mw, i0_M, i1_M,
-                                    out_wxx, blocks1d)
+            task.run(n_MG, deps_Mk, W_Mw, i0_M, i1_M, out_wxx)
 
         self.kncomm.sum(out_wxx)
 
-        if self.blockcomm.size == 1 and not wings:
+        if self.blockcomm.size == 1 and task.symmetrizable_unless_blocked:
             # Fill in upper/lower triangle also:
             nx = out_wxx.shape[1]
             il = np.tril_indices(nx, -1)
@@ -503,39 +490,6 @@ class TetrahedronIntegrator(Integrator):
             for out_xx in out_wxx:
                 out_xx[il] = out_xx[iu].conj()
 
-    def update_hilbert(self, n_MG, deps_Mk, W_Mw, i0_M, i1_M,
-                       out_wxx, blocks1d):
-        """Update output array with dissipative part."""
-        for n_G, deps_k, W_w, i0, i1 in zip(n_MG, deps_Mk, W_Mw,
-                                            i0_M, i1_M):
-            if i0 == i1:
-                continue
-
-            for iw, weight in enumerate(W_w):
-                if self.blockcomm.size > 1:
-                    myn_G = n_G[blocks1d.myslice].reshape((-1, 1))
-                    # gemm(weight, n_G.reshape((-1, 1)), myn_G,
-                    #      1.0, out_wxx[i0 + iw], 'c')
-                    mmm(weight, myn_G, 'N', n_G.reshape((-1, 1)), 'C',
-                        1.0, out_wxx[i0 + iw])
-                else:
-                    czher(weight, n_G.conj(), out_wxx[i0 + iw])
-
-    def update_hilbert_optical_limit(self, n_MG, deps_Mk, W_Mw,
-                                     i0_M, i1_M, out_wxvG):
-        """Update optical limit output array with dissipative part of the head
-        and wings."""
-        for n_G, deps_k, W_w, i0, i1 in zip(n_MG, deps_Mk, W_Mw,
-                                            i0_M, i1_M):
-            assert self.blockcomm.size == 1
-            if i0 == i1:
-                continue
-
-            for iw, weight in enumerate(W_w):
-                x_vG = np.outer(n_G[:3], n_G.conj())
-                out_wxvG[i0 + iw, 0, :, :] += weight * x_vG
-                out_wxvG[i0 + iw, 1, :, :] += weight * x_vG.conj()
-
     @timer('Get kpoint weight')
     def get_kpoint_weight(self, K, deps_k, pts_k,
                           omega_w, td):
@@ -549,3 +503,48 @@ class TetrahedronIntegrator(Integrator):
                                      W_w, omega_w, vol_s)
 
         return W_w
+
+
+class HilbertTetrahedron:
+    kind = 'spectral function'
+    symmetrizable_unless_blocked = True
+
+    def __init__(self, blockcomm):
+        self.blockcomm = blockcomm
+
+    def run(self, n_MG, deps_Mk, W_Mw, i0_M, i1_M, out_wxx):
+        """Update output array with dissipative part."""
+        blocks1d = Blocks1D(self.blockcomm, out_wxx.shape[2])
+        
+        for n_G, deps_k, W_w, i0, i1 in zip(n_MG, deps_Mk, W_Mw,
+                                            i0_M, i1_M):
+            if i0 == i1:
+                continue
+
+            for iw, weight in enumerate(W_w):
+                if blocks1d.blockcomm.size > 1:
+                    myn_G = n_G[blocks1d.myslice].reshape((-1, 1))
+                    # gemm(weight, n_G.reshape((-1, 1)), myn_G,
+                    #      1.0, out_wxx[i0 + iw], 'c')
+                    mmm(weight, myn_G, 'N', n_G.reshape((-1, 1)), 'C',
+                        1.0, out_wxx[i0 + iw])
+                else:
+                    czher(weight, n_G.conj(), out_wxx[i0 + iw])
+
+
+class HilbertOpticalLimitTetrahedron:
+    kind = 'spectral function wings'
+    symmetrizable_unless_blocked = False
+
+    def run(self, n_MG, deps_Mk, W_Mw, i0_M, i1_M, out_wxvG):
+        """Update optical limit output array with dissipative part of the head
+        and wings."""
+        for n_G, deps_k, W_w, i0, i1 in zip(n_MG, deps_Mk, W_Mw,
+                                            i0_M, i1_M):
+            if i0 == i1:
+                continue
+
+            for iw, weight in enumerate(W_w):
+                x_vG = np.outer(n_G[:3], n_G.conj())
+                out_wxvG[i0 + iw, 0, :, :] += weight * x_vG
+                out_wxvG[i0 + iw, 1, :, :] += weight * x_vG.conj()
