@@ -74,28 +74,16 @@ class KPointPair:
         return df_nm
 
 
-class PairDensityCalculator:
-    def __init__(self, gs, context, *,
-                 threshold=1, nblocks=1):
-        """Density matrix elements
-
-        Parameters
-        ----------
-        threshold : float
-            Numerical threshold for the optical limit k dot p perturbation
-            theory expansion.
-        """
+class KPointPairFactory:
+    def __init__(self, gs, context, *, nblocks=1):
         self.gs = gs
         self.context = context
 
         assert self.gs.kd.symmetry.symmorphic
 
-        self.threshold = threshold
-
         self.blockcomm, self.kncomm = block_partition(self.context.comm,
                                                       nblocks)
         self.nblocks = nblocks
-        self.ut_sKnvR = None  # gradient of wave functions for optical limit
 
         self.kptfinder = KPointFinder(self.gs.kd.bzk_kc)
         self.context.print('Number of blocks:', nblocks)
@@ -233,6 +221,20 @@ class PairDensityCalculator:
 
         return KPointPair(kpt1, kpt2, Q_G)
 
+    def pair_calculator(self):
+        # We have decoupled the actual pair density calculator
+        # from the kpoint factory, but it's still handy to
+        # keep this shortcut -- for now.
+        return ActualPairDensityCalculator(self)
+
+
+class ActualPairDensityCalculator:
+    def __init__(self, pair):
+        self.context = pair.context
+        self.blockcomm = pair.blockcomm
+        self.ut_sKnvR = None  # gradient of wave functions for optical limit
+        self.gs = pair.gs
+
     def get_optical_pair_density(self, qpd, kptpair, n_n, m_m, *,
                                  pawcorr, block=False):
         """Get the full optical pair density, including the optical limit head
@@ -379,8 +381,9 @@ class PairDensityCalculator:
 
     def calculate_optical_pair_density_head(self, n, m_m, kpt1, kpt2,
                                             block=False):
-        # Relative threshold for perturbation theory
-        threshold = self.threshold
+        # Numerical threshold for the optical limit k dot p perturbation
+        # theory expansion:
+        threshold = 1
 
         eps1 = kpt1.eps_n[n - kpt1.n1]
         deps_m = (eps1 - kpt2.eps_n)[m_m - kpt2.n1]
@@ -421,10 +424,10 @@ class PairDensityCalculator:
             # Has this chunk already been computed?
             oldchunk = any([n in chunk for chunk in degchunks_cn])
             if not oldchunk:
-                assert all([ind in n_n for ind in inds_n]), \
-                    self.context.print(
-                        '\nYou are cutting over a degenerate band ' +
-                        'using block parallelization.', inds_n, n_n)
+                if not all([ind in n_n for ind in inds_n]):
+                    raise RuntimeError(
+                        'You are cutting over a degenerate band '
+                        'using block parallelization.')
                 degchunks_cn.append((inds_n))
 
         # Calculate matrix elements by diagonalizing each block
