@@ -420,6 +420,7 @@ def get_chi_2D(omega_w=None, qpd=None, chi_wGG=None, q0=None,
     chiD_qw = np.zeros([nq, nw], dtype=complex)
     drhoM_qz = np.zeros([nq, len(z)], dtype=complex)  # induced density
     drhoD_qz = np.zeros([nq, len(z)], dtype=complex)  # induced dipole density
+
     for iq in range(nq):
         if iq != 0:
             omega_w, qpd, chi_wGG, q0 = read_chi_wGG(filenames[iq])
@@ -434,11 +435,15 @@ def get_chi_2D(omega_w=None, qpd=None, chi_wGG=None, q0=None,
         for iG in range(npw):  # List of G with Gx,Gy = 0
             if G_Gv[iG, 0] == 0 and G_Gv[iG, 1] == 0:
                 Glist.append(iG)
-
-        chiM_qw[iq] = L * chi_wGG[:, 0, 0]
-        drhoM_qz[iq] += chi_wGG[0, 0, 0]
         q_abs = np.linalg.norm(q)
         q_list_abs.append(q_abs)
+
+        # If node lacks frequency points due to block parallelization then
+        # return empty arrays
+        if nw == 0:
+            continue
+        chiM_qw[iq] = L * chi_wGG[:, 0, 0]
+        drhoM_qz[iq] += chi_wGG[0, 0, 0]
         for iG in Glist[1:]:
             G_z = G_Gv[iG, 2]
             qGr_R = np.inner(G_z, z.T).T
@@ -455,10 +460,11 @@ def get_chi_2D(omega_w=None, qpd=None, chi_wGG=None, q0=None,
                 drhoD_qz[iq, :] += 1. / L * np.exp(1j * qGr_R) * \
                     chi_wGG[0, iG, iG1] * factor1
     # Normalize induced densities with chi
-    drhoM_qz /= np.repeat(chiM_qw[:, 0, np.newaxis], drhoM_qz.shape[1],
-                          axis=1)
-    drhoD_qz /= np.repeat(chiD_qw[:, 0, np.newaxis], drhoM_qz.shape[1],
-                          axis=1)
+    if nw != 0:
+        drhoM_qz /= np.repeat(chiM_qw[:, 0, np.newaxis], drhoM_qz.shape[1],
+                              axis=1)
+        drhoD_qz /= np.repeat(chiD_qw[:, 0, np.newaxis], drhoM_qz.shape[1],
+                              axis=1)
 
     """ Returns q array, frequency array, chi2D monopole and dipole, induced
     densities and z array (all in Bohr)
