@@ -1,5 +1,3 @@
-import functools
-
 import numpy as np
 from scipy.spatial import Delaunay, cKDTree
 
@@ -32,7 +30,6 @@ class PWSymmetryAnalyzer:
     """Class for handling planewave symmetries."""
     def __init__(self, kd, qpd, context,
                  disable_point_group=False,
-                 disable_non_symmorphic=True,
                  disable_time_reversal=False):
         """Creates a PWSymmetryAnalyzer object.
 
@@ -50,8 +47,6 @@ class PWSymmetryAnalyzer:
         context: ResponseContext
         disable_point_group: bool
             Switch for disabling point group symmetries.
-        disable_non_symmorphic:
-            Switch for disabling non symmorphic symmetries.
         disable_time_reversal:
             Switch for disabling time reversal.
         """
@@ -59,13 +54,9 @@ class PWSymmetryAnalyzer:
         self.kd = kd
         self.context = context
 
-        assert disable_non_symmorphic, ('You are not allowed to use '
-                                        'non-symmorphic syms, sorry.')
-
         # Settings
         self.disable_point_group = disable_point_group
         self.disable_time_reversal = disable_time_reversal
-        self.disable_non_symmorphic = disable_non_symmorphic
         if (kd.symmetry.has_inversion or not kd.symmetry.time_reversal) and \
            not self.disable_time_reversal:
             self.context.print('\nThe ground calculation does not support time'
@@ -75,8 +66,7 @@ class PWSymmetryAnalyzer:
             self.disable_time_reversal = True
 
         self.disable_symmetries = (self.disable_point_group and
-                                   self.disable_time_reversal and
-                                   self.disable_non_symmorphic)
+                                   self.disable_time_reversal)
 
         # Number of symmetries
         U_scc = kd.symmetry.op_scc
@@ -103,10 +93,7 @@ class PWSymmetryAnalyzer:
         else:
             self.infostring += 'Time reversal included. '
 
-        if self.disable_non_symmorphic:
-            self.infostring += 'Disabled non-symmorphic symmetries. '
-        else:
-            self.infostring += 'Enabled non-symmorphic symmetries. '
+        self.infostring += 'Disabled non-symmorphic symmetries. '
 
         if self.disable_symmetries:
             self.infostring += 'All symmetries have been disabled. '
@@ -123,10 +110,8 @@ class PWSymmetryAnalyzer:
     def print_symmetries(self):
         """Handsome print function for symmetry operations."""
 
-        p = functools.partial(self.context.print, flush=False)
-
-        p()
-        nx = 6 if self.disable_non_symmorphic else 3
+        isl = ['']
+        nx = 6  # You are not allowed to use non-symmorphic syms (value 3)
         ns = len(self.s_s)
         y = 0
         for y in range((ns + nx - 1) // nx):
@@ -138,9 +123,9 @@ class PWSymmetryAnalyzer:
                     tmp = self.get_symmetry_operator(self.s_s[s])
                     op_cc, sign, TR, shift_c, ft_c = tmp
                     op_c = sign * op_cc[c]
-                    p('  (%2d %2d %2d)' % tuple(op_c), end='')
-                p()
-            self.context.print()  # flush output
+                    isl.append(f'  ({op_c[0]:2d} {op_c[1]:2d} {op_c[2]:2d})')
+                isl.append('')
+            self.context.print('\n'.join(isl))  # flush output
 
     @timer('Analyze')
     def analyze_kpoints(self):
@@ -204,8 +189,9 @@ class PWSymmetryAnalyzer:
         if self.disable_time_reversal:
             s_s = list(filter(self.is_not_time_reversal, s_s))
 
-        if self.disable_non_symmorphic:
-            s_s = list(filter(self.is_not_non_symmorphic, s_s))
+        # You are not allowed to use non-symmorphic syms, sorry. So we remove
+        # the option and always filter those symmetries out.
+        s_s = list(filter(self.is_not_non_symmorphic, s_s))
 
 #        stmp_s = []
 #        for s in s_s:
