@@ -6,6 +6,7 @@ from ase.units import Ha
 
 from gpaw import KohnShamConvergenceError
 from gpaw.convergence_criteria import check_convergence
+from gpaw.forces import calculate_forces
 
 
 class SCFLoop:
@@ -49,6 +50,7 @@ class SCFLoop:
         self.eigensolver_used = getattr(wfs.eigensolver, "name", None)
         self.check_eigensolver_state(wfs, ham, dens)
         self.niter = 1
+        converged = False
 
         while self.niter <= self.maxiter:
             self.iterate_eigensolver(wfs, ham, dens)
@@ -222,6 +224,11 @@ class SCFEvent:
         self.niter = niter
         self.log = log
 
+    def calculate_forces(self):
+        with self.wfs.timer('Forces'):
+            F_av = calculate_forces(self.wfs, self.dens, self.ham)
+        return F_av
+
 
 oops = """
 Did not converge!
