@@ -13,7 +13,7 @@ def calculate_stress(calc):
     if not isinstance(wfs, PWWaveFunctions):
         raise NotImplementedError('Calculation of stress tensor is only ' +
                                   'implemented for plane-wave mode.')
-    if ham.xc.orbital_dependent:
+    if ham.xc.orbital_dependent and ham.xc.type != 'MGGA':
         raise NotImplementedError('Calculation of stress tensor is not ' +
                                   'implemented for orbital-dependent ' +
                                   'XC functionals such as ' + ham.xc.name)
@@ -32,7 +32,7 @@ def calculate_stress(calc):
     p_G = 4 * np.pi * dens.rhot_q
     G0 = 0 if pd.gd.comm.rank > 0 else 1
     p_G[G0:] /= pd.G2_qG[0][G0:]**2
-    G_Gv = pd.get_reciprocal_vectors()
+    G_Gv = pd.get_reciprocal_vectors(add_q=False)
     for v1 in range(3):
         for v2 in range(3):
             s_vv[v1, v2] += pd.integrate(p_G, dens.rhot_q *
@@ -42,6 +42,12 @@ def calculate_stress(calc):
     s_vv -= np.eye(3) * ham.estress
     s_vv += ham.vbar.stress_tensor_contribution(dens.nt_Q)
     s_vv += dens.nct.stress_tensor_contribution(ham.vt_Q)
+    if ham.xc.type == 'MGGA':
+        nspin = ham.xc.dedtaut_sG.shape[0]
+        vtau_sQ = dens.pd2.empty((nspin,), global_array=False)
+        for s in range(nspin):
+            vtau_sQ[s] = dens.pd2.fft(ham.xc.dedtaut_sG[s])
+        s_vv += ham.xc.tauct.stress_tensor_contribution(vtau_sQ.mean(axis=0))
 
     s0 = 0.0
     s0_vv = 0.0
