@@ -1,5 +1,3 @@
-import os
-
 import pytest
 from ase import Atoms
 from ase.io import read
@@ -10,14 +8,18 @@ from gpaw import GPAW, FermiDirac
 @pytest.mark.ci
 def test_no_cell():
     with pytest.raises(ValueError):
-        H = Atoms('H', calculator=GPAW())
+        H = Atoms('H', calculator=GPAW(mode='fd'))
         H.get_potential_energy()
 
 
-@pytest.mark.skipif(os.environ.get('GPAW_NEW'), reason='WIP')
+@pytest.mark.later
 def test_read_txt(in_tmp_dir):
     a = 2.0
-    calc = GPAW(gpts=(12, 12, 12), txt='H.txt', occupations=FermiDirac(0.0))
+    calc = GPAW(
+        mode='fd',
+        gpts=(12, 12, 12),
+        txt='H.txt',
+        occupations=FermiDirac(0.0))
     H = Atoms('H',
               cell=(a, a, a),
               pbc=True,
