@@ -23,7 +23,8 @@ h2 = Atoms('H2', cell=(2 * d0, a, a), pbc=True,
 h2.center(axis=1)
 h2.center(axis=2)
 
-h2.calc = GPAW(kpts=(k, 1, 1),
+h2.calc = GPAW(mode='fd',
+               kpts=(k, 1, 1),
                occupations=FermiDirac(0.01),
                txt='h2c.txt')
 
