@@ -15,15 +15,18 @@ def test_eigen_blocked_rmm_diis(in_tmp_dir):
                    Atom('H', (x, -x, -x)),
                    Atom('H', (-x, x, -x))],
                   cell=(a, a, a), pbc=True)
-    calc = GPAW(gpts=(n, n, n), nbands=4, txt='a.txt',
-                mixer=Mixer(0.25, 3, 1), eigensolver='rmm-diis')
+    base_params = dict(
+        mode='fd',
+        gpts=(n, n, n),
+        nbands=4,
+        mixer=Mixer(0.25, 3, 1))
+    calc = GPAW(**base_params, txt='a.txt', eigensolver='rmm-diis')
     atoms.calc = calc
     e0 = atoms.get_potential_energy()
     niter0 = calc.get_number_of_iterations()
 
     es = RMMDIIS(blocksize=3)
-    calc = GPAW(gpts=(n, n, n), nbands=4, txt='b.txt',
-                mixer=Mixer(0.25, 3, 1), eigensolver=es)
+    calc = GPAW(**base_params, txt='b.txt', eigensolver=es)
     atoms.calc = calc
     e1 = atoms.get_potential_energy()
     niter1 = calc.get_number_of_iterations()
