@@ -11,17 +11,16 @@ def test_eigen_cg():
                  pbc=True)
     bulk.set_cell((d, d, a), scale_atoms=True)
     h = 0.25
-    calc = GPAW(h=h,
-                nbands=2 * 8,
-                kpts=(2, 2, 2),
-                convergence={'energy': 1e-5})
+    base_params = dict(
+        mode='fd',
+        h=h,
+        nbands=2 * 8,
+        kpts=(2, 2, 2),
+        convergence={'energy': 1e-5})
+    calc = GPAW(**base_params)
     bulk.calc = calc
     e0 = bulk.get_potential_energy()
-    calc = GPAW(h=h,
-                nbands=2 * 8,
-                kpts=(2, 2, 2),
-                convergence={'energy': 1e-5},
-                eigensolver='cg')
+    calc = GPAW(**base_params, eigensolver='cg')
     bulk.calc = calc
     e1 = bulk.get_potential_energy()
     equal(e0, e1, 5.e-5)
