@@ -12,18 +12,17 @@ def test_eigen_davidson():
                   Atom('Al', (0.5, 0.5, 0.5))], pbc=True)
     bulk.set_cell((d, d, a), scale_atoms=True)
     h = 0.25
-    calc = GPAW(h=h,
-                nbands=2 * 8,
-                kpts=(2, 2, 2),
-                convergence={'eigenstates': 7.2e-9, 'energy': 1e-5})
+    base_params = dict(
+        mode='fd',
+        h=h,
+        nbands=2 * 8,
+        kpts=(2, 2, 2))
+    base_convergence = {'eigenstates': 7.2e-9, 'energy': 1e-5}
+    calc = GPAW(**base_params, convergence=base_convergence)
     bulk.calc = calc
     e0 = bulk.get_potential_energy()
-    calc = GPAW(h=h,
-                nbands=2 * 8,
-                kpts=(2, 2, 2),
-                convergence={'eigenstates': 7.2e-9,
-                             'energy': 1e-5,
-                             'bands': 5},
+    calc = GPAW(**base_params,
+                convergence={**base_convergence, 'bands': 5},
                 eigensolver='dav')
     bulk.calc = calc
     e1 = bulk.get_potential_energy()
@@ -35,12 +34,8 @@ def test_eigen_davidson():
 
     # band parallelization
     if size % 2 == 0:
-        calc = GPAW(h=h,
-                    nbands=2 * 8,
-                    kpts=(2, 2, 2),
-                    convergence={'eigenstates': 7.2e-9,
-                                 'energy': 1e-5,
-                                 'bands': 5},
+        calc = GPAW(**base_params,
+                    convergence={**base_convergence, 'bands': 5},
                     parallel={'band': 2},
                     eigensolver=Davidson(niter=3))
         bulk.calc = calc
