@@ -63,7 +63,7 @@ def test_fileio_restart(in_tmp_dir):
     conv = {'eigenstates': 1.24, 'energy': 2e-1, 'density': 1e-1}
 
     for kwargs in [{'mode': PW(200)},
-                   {'h': 0.30}]:
+                   {'mode': 'fd', 'h': 0.30}]:
         atoms.calc = GPAW(nbands=3,
                           setups={'Na': '1'},
                           convergence=conv,
