@@ -1,6 +1,5 @@
 from math import sqrt
 
-import pytest
 from ase import Atoms
 from ase.dft.bandgap import bandgap
 
@@ -8,7 +7,6 @@ from gpaw import GPAW, FermiDirac
 from gpaw.test import equal
 
 
-@pytest.mark.later
 def test_Hubbard_U(in_tmp_dir):
     """Setup up bulk NiO in an antiferromagnetic configuration."""
     # Lattice constant:
