@@ -46,7 +46,7 @@ def get_H3(calculator=None):
 @pytest.mark.lrtddft
 def test_split(in_tmp_dir):
     fname = 'exlst.out'
-    calc = GPAW(xc='PBE', h=0.25, nbands=3, txt=fname)
+    calc = GPAW(mode='fd', xc='PBE', h=0.25, nbands=3, txt=fname)
     exlst = LrTDDFT(calc, txt=fname)
     exst = ExcitedState(exlst, 0, txt=fname)
     H2 = get_H2(exst)
@@ -67,7 +67,7 @@ def test_split(in_tmp_dir):
 def test_lrtddft_excited_state():
     txt = None
 
-    calc = GPAW(xc='PBE', h=0.25, nbands=3, spinpol=False, txt=txt)
+    calc = GPAW(mode='fd', xc='PBE', h=0.25, nbands=3, spinpol=False, txt=txt)
     H2 = get_H2(calc)
 
     xc = 'LDA'
@@ -118,7 +118,7 @@ def test_lrtddft_excited_state():
 @pytest.mark.lrtddft
 def test_io(in_tmp_dir):
     """Test output and input from files"""
-    calc = GPAW(xc='PBE', h=0.25, nbands=3, txt=None)
+    calc = GPAW(mode='fd', xc='PBE', h=0.25, nbands=3, txt=None)
     exlst = LrTDDFT(calc, txt=None)
     exst = ExcitedState(exlst, 0, txt=None)
     H2 = get_H2(exst)
@@ -156,7 +156,7 @@ def test_io(in_tmp_dir):
 @pytest.mark.lrtddft
 def test_log(in_tmp_dir):
     fname = 'ex0_silent.out'
-    calc = GPAW(xc='PBE', h=0.25, nbands=5, txt=None)
+    calc = GPAW(mode='fd', xc='PBE', h=0.25, nbands=5, txt=None)
     calc.calculate(get_H2(calc))
     exlst = LrTDDFT(calc, restrict={'eps': 0.4, 'jend': 3}, txt=None)
     exst = ExcitedState(exlst, 0, txt=fname)
@@ -173,7 +173,7 @@ def test_log(in_tmp_dir):
             assert 'Linear response TDDFT calculation' not in string
 
     fname = 'ex0_split.out'
-    calc = GPAW(xc='PBE', h=0.25, nbands=5, txt=fname)
+    calc = GPAW(mode='fd', xc='PBE', h=0.25, nbands=5, txt=fname)
     calc.calculate(get_H2(calc))
     exlst = LrTDDFT(calc, restrict={'eps': 0.4, 'jend': 3}, log=calc.log)
     exst = ExcitedState(exlst, 0, log=exlst.log, parallel=2)
@@ -201,7 +201,7 @@ def test_log(in_tmp_dir):
 @pytest.mark.lrtddft
 def test_forces():
     """Test whether force calculation works"""
-    calc = GPAW(xc='PBE', h=0.25, nbands=3, txt=None)
+    calc = GPAW(mode='fd', xc='PBE', h=0.25, nbands=3, txt=None)
     exlst = LrTDDFT(calc)
     exst = ExcitedState(exlst, 0)
     H2 = get_H2(exst)
@@ -235,7 +235,7 @@ def test_unequal_parralel_work():
     if world.size == 1:
         return
 
-    calc = GPAW(xc='PBE', charge=1, h=0.25, nbands=3, txt=None)
+    calc = GPAW(mode='fd', xc='PBE', charge=1, h=0.25, nbands=3, txt=None)
     exlst = LrTDDFT(calc, txt=None)
     H3 = get_H3()
 
