@@ -34,7 +34,8 @@ def test_ofdft_ofdft(in_tmp_dir):
                      cell=(a, a, a))
 
         mixer = Mixer(0.3, 5, 1)
-        calc = GPAW(gpts=(32, 32, 32),
+        calc = GPAW(mode='fd',
+                    gpts=(32, 32, 32),
                     txt='-',
                     xc=xcname,
                     setups=setups,
