@@ -1,4 +1,4 @@
-# http://listserv.fysik.dtu.dk/pipermail/gpaw-developers/2014-February/004374.html
+# listserv.fysik.dtu.dk/pipermail/gpaw-developers/2014-February/004374.html
 from gpaw import GPAW
 from gpaw.test import equal
 from ase.build import molecule
@@ -9,12 +9,8 @@ def test_pathological_LDA_unstable():
         mol = molecule('H2')
         mol.center(vacuum=1.5)
         calc = GPAW(h=0.3, nbands=2, mode='lcao', txt=None, basis='sz(dzp)',
-                    xc='oldLDA')
-
-        def stop():
-            calc.scf.converged = True
-
-        calc.attach(stop, 1)
+                    xc='oldLDA',
+                    convergence={'maximum iterations': 1})
         mol.calc = calc
         e = mol.get_potential_energy()
         if i == 0:
