@@ -21,12 +21,12 @@ from gpaw.test.conftest import response_band_cutoff
 def generate_system_s(spincomponents=['00', '+-']):
     # Compute chiks for different materials and spin components
     system_s = [  # wfs, spincomponent
-        ('fancy_si_pw_wfs', '00'),
-        ('al_pw_wfs', '00'),
-        ('fe_pw_wfs', '00'),
-        ('fe_pw_wfs', '+-'),
-        ('co_pw_wfs', '00'),
-        ('co_pw_wfs', '+-'),
+        ('fancy_si_pw', '00'),
+        ('al_pw', '00'),
+        ('fe_pw', '00'),
+        ('fe_pw', '+-'),
+        ('co_pw', '00'),
+        ('co_pw', '+-'),
     ]
 
     # Filter spincomponents
@@ -43,13 +43,13 @@ def generate_qrel_q():
 
 
 def get_q_c(wfs, qrel):
-    if wfs in ['fancy_si_pw_wfs', 'al_pw_wfs']:
+    if wfs in ['fancy_si_pw', 'al_pw']:
         # Generate points on the G-X path
         q_c = qrel * np.array([1., 0., 1.])
-    elif wfs == 'fe_pw_wfs':
+    elif wfs == 'fe_pw':
         # Generate points on the G-N path
         q_c = qrel * np.array([0., 0., 1.])
-    elif wfs == 'co_pw_wfs':
+    elif wfs == 'co_pw':
         # Generate points on the G-M path
         q_c = qrel * np.array([1., 0., 0.])
     else:
@@ -65,10 +65,14 @@ def get_tolerances(system, qrel):
 
     # Si and Fe the density-density response has perfect symmetry
     atols = {
-        'fancy_si_pw_wfs_00': 1e-8,
-        'fe_pw_wfs_00': 1e-8,
+        'fancy_si_pw_00': 1e-8,
+        'fe_pw_00': 1e-8,
     }
 
+    # For the rest, we need to adjust the absolute tolerances. In general
+    # it should be possible to lower these tolerances when increasing the
+    # number of bands.
+
     # For Al, the symmetries are not perfectly conserved, but worst for the
     # q-point q_X
     if qrel == 0.0:
@@ -77,17 +81,17 @@ def get_tolerances(system, qrel):
         al_atol = 5e-5
     elif qrel == 0.5:
         al_atol = 2e-4
-    atols['al_pw_wfs_00'] = al_atol
+    atols['al_pw_00'] = al_atol
 
     # For Fe, the symmetries are not perfectly conserved for the
     # transverse magnetic response
     if qrel == 0.0:
-        fet_atol = 2e-3
+        fet_atol = 3e-3
     elif qrel == 0.25:
-        fet_atol = 8e-3
+        fet_atol = 16e-3
     elif qrel == 0.5:
         fet_atol = 5e-4
-    atols['fe_pw_wfs_+-'] = fet_atol
+    atols['fe_pw_+-'] = fet_atol
 
     # For the density-density reponse in Co, the symmetries are not perfectly
     # conserved for any of the q-points, but quite well conserved for q = 0
@@ -97,18 +101,18 @@ def get_tolerances(system, qrel):
         co_atol = 5e-3
     elif qrel == 0.5:
         co_atol = 1e-3
-    atols['co_pw_wfs_00'] = co_atol
+    atols['co_pw_00'] = co_atol
 
     # For the transverse magnetic response in Co, the symmetries are not
     # perfectly conserved for any of the q-points, but again quite well
     # conserved for q = 0
     if qrel == 0.0:
-        cot_atol = 5e-5
+        cot_atol = 5e-4
     elif qrel == 0.25:
-        cot_atol = 5e-4
+        cot_atol = 1e-3
     elif qrel == 0.5:
-        cot_atol = 5e-4
-    atols['co_pw_wfs_+-'] = cot_atol
+        cot_atol = 1e-3
+    atols['co_pw_+-'] = cot_atol
 
     if identifier not in atols.keys():
         raise ValueError(system, qrel)
@@ -164,7 +168,10 @@ def test_chiks(in_tmp_dir, gpw_files, system, qrel, gammacentered):
     ecut = 50
     # Test vanishing and finite real and imaginary frequencies
     frequencies = np.array([0., 0.05, 0.1, 0.2])
-    complex_frequencies = list(frequencies + 0.j) + list(frequencies + 0.1j)
+
+    # We add a small (1e-6j) imaginary part to avoid risky floating point
+    # operations that may cause NaNs or divide-by-zero.
+    complex_frequencies = list(frequencies + 1e-6j) + list(frequencies + 0.1j)
     zd = ComplexFrequencyDescriptor.from_array(complex_frequencies)
 
     # Part 2: Check toggling of calculation parameters
@@ -289,8 +296,8 @@ def test_chiks_vs_chi0(in_tmp_dir, gpw_files, system, qrel):
                      frequencies=frequencies, eta=eta,
                      ecut=ecut, nbands=nbands,
                      hilbert=False, intraband=False)
-    chi0_data = chi0_calc.calculate(q_c)
-    chi0_wGG = chi0_data.get_distributed_frequencies_array()
+    chi0 = chi0_calc.calculate(q_c)
+    chi0_wGG = chi0.body.get_distributed_frequencies_array()
 
     # Part 3: Check chiks vs. chi0
     assert chiks.array == pytest.approx(chi0_wGG, rel=rtol, abs=atol)
