@@ -21,7 +21,7 @@ def test_restart_file(in_tmp_dir, gpw_files, needs_ase_master):
                   nblocks=world.size,
                   ecut=40,
                   kpts=[0, 1])
-    gw = FragileG0W0(gpw_files['bn_pw_wfs'], **kwargs)
+    gw = FragileG0W0(gpw_files['bn_pw'], **kwargs)
     with pytest.raises(ValueError, match='Cthulhu*'):
         gw.calculate()
 
@@ -31,10 +31,10 @@ def test_restart_file(in_tmp_dir, gpw_files, needs_ase_master):
     # The FragileG0W0 cannot by itself calculate the full thing because
     # calculate_q is called 16 times in total. Thus, it must be that
     # it was helped by the previous calculation.
-    gw = FragileG0W0(gpw_files['bn_pw_wfs'], **kwargs)
+    gw = FragileG0W0(gpw_files['bn_pw'], **kwargs)
     results = gw.calculate()
 
-    gw = G0W0(gpw_files['bn_pw_wfs'], filename='referencecalc', **kwargs)
+    gw = G0W0(gpw_files['bn_pw'], filename='referencecalc', **kwargs)
     results2 = gw.calculate()
 
     assert np.allclose(results['qp'], results2['qp'])
