@@ -14,7 +14,7 @@ def test_gw_spinpol(in_tmp_dir, gpw_files, needs_ase_master):
     else:
         nblocks = 1
 
-    gw = G0W0(gpw_files['h2_bcc_afm_wfs'],
+    gw = G0W0(gpw_files['h2_bcc_afm'],
               nbands=4,  # keep consistent with gpw nbands
               ecut=100,
               kpts=[(0, 0, 0)],
