@@ -7,10 +7,13 @@ from ase.parallel import parprint
 
 from gpaw import GPAW, PW
 from gpaw.test import findpeak, equal
-from gpaw.response.susceptibility import FourComponentSusceptibilityTensor
-from gpaw.response.susceptibility import read_component
 from gpaw.mpi import size, world
 
+from gpaw.response import ResponseGroundStateAdapter
+from gpaw.response.chiks import ChiKSCalculator
+from gpaw.response.susceptibility import ChiFactory
+from gpaw.response.pair_functions import read_susceptibility_array
+
 
 @pytest.mark.kspair
 @pytest.mark.response
@@ -52,23 +55,13 @@ def test_response_two_aluminum_chi_RPA(in_tmp_dir):
     q2_qc = [np.array([1 / 4., 0., 0.]), np.array([- 1 / 4., 0., 0.])]
     w = np.linspace(0, 24, 241)
 
-    # Calculate susceptibility using Al
-    fcst = FourComponentSusceptibilityTensor(calc1, fxc='RPA',
-                                             eta=0.2, ecut=50)
-    for q, q_c in enumerate(q1_qc):
-        fcst.get_component_array('00', q_c, w, array_ecut=25,
-                                 filename='Al1_chiGG_q%d.pckl' % (q + 1))
-        world.barrier()
+    # Calculate susceptibility using Al1
+    calculate_chi(calc1, q1_qc, w, filename_prefix='Al1')
 
     t4 = time.time()
 
     # Calculate susceptibility using Al2
-    fcst = FourComponentSusceptibilityTensor(calc2, fxc='RPA',
-                                             eta=0.2, ecut=50)
-    for q, q_c in enumerate(q2_qc):
-        fcst.get_component_array('00', q_c, w, array_ecut=25,
-                                 filename='Al2_chiGG_q%d.pckl' % (q + 1))
-        world.barrier()
+    calculate_chi(calc2, q2_qc, w, filename_prefix='Al2')
 
     t5 = time.time()
 
@@ -81,10 +74,10 @@ def test_response_two_aluminum_chi_RPA(in_tmp_dir):
     # Check that results are consistent, when structure is simply repeated
 
     # Read results
-    w11_w, G11_Gc, chiks11_wGG, chi11_wGG = read_component('Al1_chiGG_q1.pckl')
-    w21_w, G21_Gc, chiks21_wGG, chi21_wGG = read_component('Al2_chiGG_q1.pckl')
-    w12_w, G12_Gc, chiks12_wGG, chi12_wGG = read_component('Al1_chiGG_q2.pckl')
-    w22_w, G22_Gc, chiks22_wGG, chi22_wGG = read_component('Al2_chiGG_q2.pckl')
+    w11_w, G11_Gc, chi11_wGG = read_susceptibility_array('Al1_chiGG_q1.pckl')
+    w21_w, G21_Gc, chi21_wGG = read_susceptibility_array('Al2_chiGG_q1.pckl')
+    w12_w, G12_Gc, chi12_wGG = read_susceptibility_array('Al1_chiGG_q2.pckl')
+    w22_w, G22_Gc, chi22_wGG = read_susceptibility_array('Al2_chiGG_q2.pckl')
 
     # Check that reciprocal lattice vectors remain as assumed in check below
     equal(np.linalg.norm(G11_Gc[0]), 0., 1e-6)
@@ -101,3 +94,24 @@ def test_response_two_aluminum_chi_RPA(in_tmp_dir):
     wpeak22, Ipeak22 = findpeak(w22_w, -chi22_wGG[:, 1, 1].imag)
     equal(wpeak12, wpeak22, 0.05)
     equal(Ipeak12, Ipeak22, 1.0)
+
+
+def calculate_chi(calc, q_qc, w,
+                  eta=0.2, ecut=50,
+                  spincomponent='00', fxc='RPA',
+                  filename_prefix=None, reduced_ecut=25):
+    gs = ResponseGroundStateAdapter(calc)
+    chiks_calc = ChiKSCalculator(gs, ecut=ecut)
+    chi_factory = ChiFactory(chiks_calc)
+
+    if filename_prefix is None:
+        filename = 'chiGG_qXXX.pckl'
+    else:
+        filename = filename_prefix + '_chiGG_qXXX.pckl'
+
+    for q, q_c in enumerate(q_qc):
+        fname = filename.replace('XXX', str(q + 1))
+        _, chi = chi_factory(spincomponent, q_c, w + 1.j * eta, fxc=fxc)
+        chi = chi.copy_with_reduced_ecut(reduced_ecut)
+        chi.write_array(fname)
+        world.barrier()
