@@ -7,6 +7,7 @@ from gpaw.xc.rpa import RPACorrelation
 from gpaw.xc.fxc import FXCCorrelation
 
 
+@pytest.mark.rpa
 @pytest.mark.response
 def test_rpa_rpa_energy_Ni(in_tmp_dir):
     Ni = bulk('Ni', 'fcc')
@@ -21,11 +22,12 @@ def test_rpa_rpa_energy_Ni(in_tmp_dir):
     Ni.get_potential_energy()
     calc.diagonalize_full_hamiltonian(nbands=50)
 
-    rpa = RPACorrelation(calc, nfrequencies=8, skip_gamma=True)
-    E_rpa = rpa.calculate(ecut=[50])
+    rpa = RPACorrelation(calc, nfrequencies=8, skip_gamma=True, ecut=[50])
+    E_rpa = rpa.calculate()
 
-    fxc = FXCCorrelation(calc, nlambda=16, nfrequencies=8, skip_gamma=True)
-    E_fxc = fxc.calculate(ecut=[50])
+    fxc = FXCCorrelation(calc, nlambda=16, nfrequencies=8, skip_gamma=True,
+                         ecut=[50])
+    E_fxc = fxc.calculate()
 
     equal(E_rpa, -7.826, 0.01)
     equal(E_fxc, -7.826, 0.01)
