@@ -1,5 +1,3 @@
-from ase import Atoms
-from gpaw import GPAW
 from gpaw.tddft import TDDFT
 from gpaw.tddft.ehrenfest import EhrenfestVelocityVerlet
 from gpaw.test import equal
@@ -7,19 +5,9 @@ import pytest
 
 
 @pytest.mark.later
-def test_tddft_ehrenfest_nacl(in_tmp_dir):
-    d = 4.0
-    atoms = Atoms('NaCl', [(0, 0, 0), (0, 0, d)])
-    atoms.center(vacuum=4.5)
+def test_tddft_ehrenfest_nacl(in_tmp_dir, gpw_files):
 
-    gs_calc = GPAW(nbands=4, eigensolver='cg', gpts=(32, 32, 44), xc='LDA',
-                   setups={'Na': '1'})
-    atoms.calc = gs_calc
-    atoms.get_potential_energy()
-
-    gs_calc.write('nacl_gs.gpw', 'all')
-
-    td_calc = TDDFT('nacl_gs.gpw', propagator='EFSICN')
+    td_calc = TDDFT(gpw_files['nacl_fd'], propagator='EFSICN')
     evv = EhrenfestVelocityVerlet(td_calc, 0.001)
 
     i = 0
