@@ -9,7 +9,7 @@ def test_broydenmixer(in_tmp_dir):
     bulk = Atoms([Atom('Li')], pbc=True, cell=(a, a, a))
     k = 2
     g = 16
-    calc = GPAW(gpts=(g, g, g), kpts=(k, k, k), nbands=2,
+    calc = GPAW(mode='fd', gpts=(g, g, g), kpts=(k, k, k), nbands=2,
                 mixer=BroydenMixer())
     bulk.calc = calc
     e = bulk.get_potential_energy()
