@@ -9,7 +9,8 @@ from gpaw.test import equal
 
 
 def test_fermisplit(in_tmp_dir):
-    calc = GPAW(occupations=FermiDirac(width=0.1, fixmagmom=True),
+    calc = GPAW(mode='fd',
+                occupations=FermiDirac(width=0.1, fixmagmom=True),
                 mixer=MixerSum(beta=0.05, nmaxold=3, weight=50.0),
                 convergence={'energy': 0.1, 'eigenstates': 1.5e-1,
                              'density': 1.5e-1})
