@@ -16,19 +16,21 @@ def test_fixocc():
     H2.minimal_box(box, h)
     convergence = {'energy': 0.01, 'eigenstates': 0.001, 'density': 0.01}
 
+    base_kwargs = dict(
+        mode='fd',
+        h=h,
+        nbands=nbands,
+        convergence=convergence,
+        txt=txt)
+
     if 1:
         # test ZeroKelvin vs FixedOccupations
-        c = GPAW(h=h, nbands=nbands,
-                 occupations={'width': 0.0},
-                 convergence=convergence,
-                 txt=txt)
+        c = GPAW(**base_kwargs, occupations={'width': 0.0})
         H2.calc = c
         E_zk = H2.get_potential_energy()
 
-        c = GPAW(h=h, nbands=nbands,
-                 occupations=dict(name='fixed', numbers=[[1, 0]]),
-                 convergence=convergence,
-                 txt=txt)
+        c = GPAW(**base_kwargs,
+                 occupations=dict(name='fixed', numbers=[[1, 0]]))
         H2.calc = c
         E_fo = H2.get_potential_energy()
         parprint(E_zk, E_fo)
@@ -36,17 +38,14 @@ def test_fixocc():
 
     if 1:
         # test spin-paired vs spin-polarized
-        c = GPAW(h=h, nbands=nbands,
-                 occupations={'name': 'fixed', 'numbers': [[0.5, 0.5]]},
-                 convergence=convergence,
-                 txt=txt)
+        c = GPAW(**base_kwargs,
+                 occupations={'name': 'fixed', 'numbers': [[0.5, 0.5]]})
         H2.calc = c
         E_ns = H2.get_potential_energy()
     if 1:
-        c = GPAW(h=h, nbands=nbands, spinpol=True,
-                 occupations={'name': 'fixed', 'numbers': [[0.5, 0.5]] * 2},
-                 convergence=convergence,
-                 txt=txt)
+        c = GPAW(**base_kwargs,
+                 spinpol=True,
+                 occupations={'name': 'fixed', 'numbers': [[0.5, 0.5]] * 2})
         H2.calc = c
         E_sp = H2.get_potential_energy()
         parprint(E_ns, E_sp)
