@@ -24,7 +24,8 @@ def test_jellium(in_tmp_dir):
 
     surf = Atoms(pbc=(True, True, False),
                  cell=(a, a, v + L + v))
-    surf.calc = GPAW(background_charge=bc,
+    surf.calc = GPAW(mode='fd',
+                     background_charge=bc,
                      poissonsolver={'dipolelayer': 'xy'},
                      xc='LDA_X+LDA_C_WIGNER',
                      eigensolver='dav',
