@@ -8,12 +8,13 @@ def test_xc_nonselfconsistentLDA(in_tmp_dir):
     a = 7.5 * Bohr
     n = 16
     atoms = Atoms([Atom('He', (0.0, 0.0, 0.0))], cell=(a, a, a), pbc=True)
-    calc = GPAW(gpts=(n, n, n), nbands=1, xc='LDA')
+    calc = GPAW(mode='fd', gpts=(n, n, n), nbands=1, xc='LDA')
     atoms.calc = calc
     e1 = atoms.get_potential_energy()
     e1ref = calc.get_reference_energy()
     de12 = calc.get_xc_difference({'name': 'PBE', 'stencil': 1})
-    calc = GPAW(gpts=(n, n, n), nbands=1, xc={'name': 'PBE', 'stencil': 1})
+    calc = GPAW(
+        mode='fd', gpts=(n, n, n), nbands=1, xc={'name': 'PBE', 'stencil': 1})
     atoms.calc = calc
     e2 = atoms.get_potential_energy()
     e2ref = calc.get_reference_energy()
